#include <stdlib.h>
#include  <string.h>
#include "util.h"
#include "etable.h"
#include "board.h"


etable_t ETable[1];

void ETableInit(etable_t *etable){
    etable->Slot=NULL;
    etable->Size=0;
    etable->EHashMask=0;
    ETableResetStats(etable);
}

void ETableClear(etable_t *etable){
    if(etable->Slot!=NULL){
	free(etable->Slot);
    }
    etable->Slot=NULL;
}

void ETableZero (etable_t *etable){
    memset (etable->Slot, 0, etable->Size * sizeof (etable_slot_t));
}

void ETableResetStats(etable_t *etable){
    etable->TotalEvalHashCnt = 0;
    etable->GoodEvalHashCnt = 0;
}

void ETableSetSize(etable_t *etable, unsigned int size){
    unsigned int esize,i;
    ETableClear(etable);
    i = (size < EVALSLOTS ? EVALSLOTS : size);
    etable->EHashMask = 0;
    while ((i>>=1) > 0)	{
	etable->EHashMask <<= 1;
	etable->EHashMask |= 1;
    }
    etable->Size = (etable->EHashMask+1);
    etable->Slot = (etable_slot_t *) malloc (etable->Size * sizeof (etable_slot_t));
    if (etable->Slot == NULL) {
	OutputConsole("Not enough memory for eval cache, goodbye.\n");
	exit(EXIT_FAILURE);
    } else {
	esize = (etable->Size *sizeof (etable_slot_t)) >> 10;
    }

    ETableZero (etable);

    OutputConsole("Eval cache table: Entries=%dK Size=%dK\n",
		 etable->Size  >> 10, esize);
    


}

void ETableSetSizeMb(etable_t *etable, unsigned int size){
    int slotsize;
    slotsize=(1048576*size)/(sizeof(etable_slot_t));
    ETableSetSize(etable,slotsize);
}

etable_slot_t * ETableGet(etable_t *etable, 
			  board_t *board){
    etable_slot_t *p;
    etable->TotalEvalHashCnt++;
    p=etable->Slot + (board->HashKey & etable->EHashMask);
    // sacrifice a few cycles for statistics.
    if(EVAL_VALID(board,p)){
	++etable->GoodEvalHashCnt;
    }
    return p;
}

void ETableStatsShow(etable_t *etable){
    OutputConsole("EHash: Get=%uk Good=%2.0f%%\n",
		  etable->TotalEvalHashCnt/1000,
		  100*((double) etable->GoodEvalHashCnt)/((double) etable->TotalEvalHashCnt));
}

