/* GNU Chess 5.0 - eval.c - evaluation code
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   GNU Chess is based on the two research programs 
   Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.

   GNU Chess is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GNU Chess is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info: 
     bug-gnu-chess@gnu.org
     cracraft@ai.mit.edu, cracraft@stanfordalumni.org, cracraft@earthlink.net
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "piece.h"
#include "util.h"
#include "eval.h"
#include "bitboard.h"
#include "move.h"
#include "material.h"
#include "square.h"
#include "ptable.h"
#include "searchdata.h"
#include "searchstats.h"
#include "attack.h"
#include "swap.h"
#include "hung.h"
#include "endgame.h"
#include "gtb.h"
#include "random.h"
#include "search.h"
#include "pin.h"
#include "kpk.h"
#include "kppp_k.h"
#include "kppp_kpp_.h"
#include "log.h"
#include "magic.h"
#include "etable.h"
#include "krpkr.h"
#include "krkp.h"


int LoneKing (board_t *board, int side, int loser);
int ScoreKBNK (board_t *board, int side, int loser);
int BishopTrapped (board_t *board, short side);
int DoubleQR7 (board_t *board, short side);
int ScoreKBPK(board_t *board, uint32_t *flags);

int t_dummy=0;

// fruit (temporary)
static const int KingAttackWeight[16] = {
    0, 0, 128, 192, 224, 240, 248, 252, 254, 255, 256, 256 ,256, 256, 256, 256,
};


/* PST's for pawns */
/*
static int PawnSq[2][64] = 
{
{  0,  0,  0,  0,  0,  0,  0,  0,
   5,  5,  5,-10,-10,  5,  5,  5,
  -2, -2, -2,  6,  6, -2, -2, -2,
   0,  0,  0, 25, 25,  0,  0,  0,
   2,  2, 12, 16, 16, 12,  2,  2,
   4,  8, 12, 16, 16, 12,  4,  4,
   4,  8, 12, 16, 16, 12,  4,  4,
   0,  0,  0,  0,  0,  0,  0,  0},
{  0,  0,  0,  0,  0,  0,  0,  0,
   4,  8, 12, 16, 16, 12,  4,  4,
   4,  8, 12, 16, 16, 12,  4,  4,
   2,  2, 12, 16, 16, 12,  2,  2,
   0,  0,  0, 25, 25,  0,  0,  0,
  -2, -2, -2,  6,  6, -2, -2, -2,
   5,  5,  5,-10,-10,  5,  5,  5,
   0,  0,  0,  0,  0,  0,  0,  0}
};
*/
/* PST's for King */
 /*
static const int KingSq[64] =
{
   24, 24, 24, 16, 16,  0, 32, 32,
   24, 20, 16, 12, 12, 16, 20, 24,
   16, 12,  8,  4,  4,  8, 12, 16,
   12,  8,  4,  0,  0,  4,  8, 12,
   12,  8,  4,  0,  0,  4,  8, 12,
   16, 12,  8,  4,  4,  8, 12, 16,
   24, 20, 16, 12, 12, 16, 20, 24,
   24, 24, 24, 16, 16,  0, 32, 32
};
 */
static const int EndingKing[64] =
{
   0,  6, 12, 18, 18, 12,  6,  0,
   6, 12, 18, 24, 24, 18, 12,  6,
  12, 18, 24, 32, 32, 24, 18, 12,
  18, 24, 32, 48, 48, 32, 24, 18,
  18, 24, 32, 48, 48, 32, 24, 18,
  12, 18, 24, 32, 32, 24, 18, 12,
   6, 12, 18, 24, 24, 18, 12,  6,
   0,  6, 12, 18, 18, 12,  6,  0
};





/* This is the infamous quad function from Fruit 2.1 */
static const int Quad[2][8] =
 { { 0, 20, 20, 32, 56, 92, 140, 0}, {0, 140, 92, 56, 32, 20, 20, 0} };

/* penalties for distance of king from pawns */
static const int dist_penalty[8] = {
    0,4,6,7,8,9,10,10
};

/* Penalties for one or more isolated pawns on a given file */
static const int isolani_normal[8] = {
  -8, -10, -12, -14, -14, -12, -10, -8
};
/* Penalties if the file is half-open (i.e. no enemy pawns on it) */
static const int isolani_weaker[8] = {
  -22, -24, -26, -28, -28, -26, -24, -22
};

/* Recognizers for outposts */
static const int MinorOutpost[2][64] =
{
  { 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 1, 1, 1, 1, 0, 0,
    0, 1, 1, 1, 1, 1, 1, 0,
    0, 0, 1, 1, 1, 1, 0, 0,
    0, 0, 0, 1, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 1, 1, 0, 0, 0,
    0, 0, 1, 1, 1, 1, 0, 0,
    0, 1, 1, 1, 1, 1, 1, 0,
    0, 0, 1, 1, 1, 1, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0 }
};


/* bonusses for pawns in pawn cover */
static const int pawncover[9] = { -60, -30, 0, 5, 30, 30, 30, 30, 30 };

/* king safety factor (indexed by phase) */
static const int factor[9] = { 7, 8, 8, 7, 6, 5, 4, 2, 0, };



void EvalInit(option_list_t *options){
    enum Piece piece;
    int side;
    int rank;
    evals.stm_bonusO=20;
    evals.stm_bonusE=10;
    evals.Value[empty]=0;
    evals.ValueP=100;
    evals.Value[pawn]=evals.ValueP;
    evals.ValueN=350;
    evals.Value[knight]=evals.ValueN;
    evals.ValueB=350;
    evals.Value[bishop]=evals.ValueB;
    evals.ValueR=550;
    evals.Value[rook]=evals.ValueR;
    evals.ValueQ=1100;
    evals.Value[queen]=evals.ValueQ;
    evals.ValueK=2000;
    evals.Value[king]=evals.ValueK;

    for(piece=pawn; piece<=king;piece++){
	evals.ValueO[piece]=evals.Value[piece];
	evals.ValueE[piece]=evals.Value[piece];
    }

    evals.ValueO[pawn]=100;

    evals.ValueO[knight]=340;
    evals.ValueO[bishop]=363;
    evals.ValueO[rook]=585;
    evals.ValueO[queen]=1156;

    for(piece=pawn; piece<=king;piece++){
	evals.ValueE[piece]=evals.ValueO[piece];
    }

    //    evals.ValueO[pawn]=95;
    //    evals.ValueE[pawn]=114;

    evals.ValueO[pawn]=97;
    evals.ValueE[pawn]=112;

    //    evals.ValueO[pawn]=OptionGetInt(Option,"T3");
    //    evals.ValueE[pawn]=OptionGetInt(Option,"T4");
    evals.ValueO[pawn]=107;
    evals.ValueE[pawn]=106;


    evals.ValueO[bishop]=387;
    evals.ValueE[bishop]=368;

    evals.ValueO[knight]=357;
    evals.ValueE[knight]=342;

    evals.ValueO[rook]=497;
    evals.ValueE[rook]=631;

    evals.ValueO[queen]=1279;
    evals.ValueE[queen]=1087;


    evals.tradepiece=4;
    evals.tradepawns=2;
    evals.hungpenalty=-20;
    evals.rookmoved=-20;
    evals.kingmoved=-20;
    evals.notcastled=-8;
    evals.pfactor=550;
    evals.earlyminorrepeat=-7;
    evals.undevelopedminor=-8;
    
    evals.eight_pawns=-10;
    evals.stonewall=-10;
    evals.lockedpawns=-10;
    evals.earlywingpawnmove=-6;
    evals.earlycenterrepeat=-6;
    evals.pawnbaseattack=-18;
    evals.blockedpawn=-48;
    evals.connectedpp=50;
    evals.pawnnearking=40;
    evals.fianchetto_target=-13;
    
    evals.knightonrim=-13;
    evals.outpostknight=10;
    evals.pinnedknight=-30;
    evals.knighttrapped=-250;

    evals.doubledbishops=32;
    evals.outpostbishop=8;
    evals.fianchetto=8;
    evals.goodendingbishop=16;
    evals.bishoptrapped=-250;
    evals.pinnedbishop=-30;

    evals.rook7rank=30;
    evals.rooks7rank=30;
    evals.rookhalffile=5;
    evals.rookopenfile=6;
    evals.rookbehindpp=6;
    evals.rookinfrontpp=-10;
    evals.pinnedrook=-50;
    evals.rooktrapped=-10;
    evals.rookliberated=40;

    evals.earlyqueenmove=-40;
    evals.queennearking=12;
    evals.pinnedqueen=-90;

    evals.kingopenfile=-10;
    evals.kingopenfile1=-6;
    evals.attackking=10;
    evals.defendking=6;
    evals.hopen=-30;
    evals.gopen=-30;
    evals.king_defender_deficit=-50;
    evals.king_back_rank_weak=-40;
    evals.rupture=-20;

    evals.doubled_pawn=-13;

    // These values were found by clop. They seem rather ridiculous.
    // At the very least we must change the bonus if there are no
    // major pieces on the board.

    evals.isolated_pawn_open=-26;
    evals.isolated_doubled_pawn_open=-24;
    evals.isolated_pawn_normal=-7;
    evals.isolated_doubled_pawn_normal=-12;
    evals.pawn_endgame_bonus=22;

    //    evals.passer_seedO=OptionGetInt(Option,"T1");
    //    evals.passer_seedE=OptionGetInt(Option,"T2");
    evals.passer_seedO=23;
    evals.passer_seedE=82;

    for(side=0;side<=1;side++){
	for(rank=0;rank<=7;rank++){
	    evals.passerO[side][rank]=(Quad[side][rank]*evals.passer_seedO)>>6;
	    evals.passerE[side][rank]=(Quad[side][rank]*evals.passer_seedE)>>6;
	}
    }

    evals.free_passer_seedO=0;
    evals.free_passer_seedE=79;

    for(side=0;side<=1;side++){
	for(rank=0;rank<=7;rank++){
	    evals.free_passerO[side][rank]=(Quad[side][rank]*evals.free_passer_seedO)>>6;
	    evals.free_passerE[side][rank]=(Quad[side][rank]*evals.free_passer_seedE)>>6;
	}
    }


    evals.mobilityO[king]=0;
    evals.mobilityO[queen]=53;
    evals.mobilityO[rook]=77;
    evals.mobilityO[bishop]=100;
    evals.mobilityO[knight]=97;

    evals.mobilityE[king]=0;
    evals.mobilityE[queen]=117;
    evals.mobilityE[rook]=115;
    evals.mobilityE[bishop]=78;
    evals.mobilityE[knight]=82;

    evals.centercontrol[king]=0;
    evals.centercontrol[queen]=0;
    evals.centercontrol[rook]=16;
    evals.centercontrol[bishop]=80;
    evals.centercontrol[knight]=68;

    evals.attackenemyking[king]=0;
    evals.attackenemyking[queen]=39;
    evals.attackenemyking[rook]=43;
    evals.attackenemyking[knight]=31;

    evals.mobilityO[bishop]=28;
    evals.mobilityE[bishop]=40;
    evals.centercontrol[bishop]=126;
    evals.attackenemyking[bishop]=24;


}

void EvalDataInit(evaldata_t *evaldata){
   memset(evaldata,0,sizeof(evaldata_t));
}

int ScoreP (board_t *board, 
	    short side,
	    evaldata_t *evaldata)
/***************************************************************************
 *
 *  Pawn evaluation is based on the following factors (which is being
 *  constantly updated!).
 *
 *  1.  Pawn square tables.
 *  2.  Passed pawns.
 *  3.  Backward pawns.
 *  4.  Pawn base under attack.
 *  5.  Doubled pawns 
 *  6.  Isolated pawns 
 *  7.  Connected passed pawns on 6/7th rank.
 *  8.  Unmoved & blocked d, e pawn
 *  9.  Passed pawn which cannot be caught.
 *  10. Pawn storms.
 *
 ***************************************************************************/
{
   int xside;
   int s, sq, i, i1;
   int n1, n2;
   BitBoard backward;
   int nfile[8];
   int EnemyKing;
   BitBoard my_pawns, t, p, his_pawns, blocker, *e, pieces[2]={0,0};
   int n;
   int ph;
   BitBoard passed;
   BitBoard weak;
   BitBoard attacked;
   BitBoard blocked;
   int phase;
   int doubled_pawn_cnt;
   int my_pawn_cnt;
   int real_pawn_cnt;
   ptable_slot_t *pawns;

   if (board->b[side][pawn] == NULLBITBOARD)
      return (0);
   xside = 1^side;
   EnemyKing = board->king[xside];
   his_pawns = board->b[xside][pawn];
   my_pawns  = board->b[side][pawn];

   pawns=PTableGet(PTable, 
		   board,
		   side);



   if(PAWN_VALID(board,pawns)){
       passed=evaldata->passed[side]=pawns->passed;
       weak=evaldata->weak[side]=pawns->weak;
       backward=evaldata->backward[side]=pawns->backward;
       attacked=evaldata->attacked[side]=pawns->attacked;
       blocked=evaldata->blocked[side]=pawns->blocked;
       evaldata->real_pawns[side]=pawns->real_pawns;
       s=pawns->score;
       phase=board->phase;
       goto phase2;
   }
   

   s = 0;
   doubled_pawn_cnt=0;
   my_pawn_cnt=nbits(my_pawns);
   passed = NULLBITBOARD;
   weak = NULLBITBOARD;
   attacked = NULLBITBOARD;
   backward = NULLBITBOARD;

   blocked = my_pawns & (side==white?his_pawns<<8:his_pawns>>8);

   memset (nfile, 0, sizeof (nfile));

   t=my_pawns;
   
   while (t) {
      sq = leadz (t);
      CLEARBIT (t, sq);

      attacked|=bitboards.MoveArray[ptype[side]][sq];
      
      //      s += PawnSq[side][sq]; 

      /*  Passed pawns  */
      if (!(his_pawns & bitboards.PassedPawnMask[side][sq]) && 
	  !(my_pawns & bitboards.Front[side][sq])){
            passed |= bitboards.BitPosArray[sq];
	    //	    s += (Quad[side][RANK(sq)] * MAX(3,board->phase)) / 6;
	    s += PHASEMIX(board,evals.passerO[side][RANK(sq)],evals.passerE[side][RANK(sq)]);
      }
      
      /*  Backward pawns */
      // i=square in front of pawn.
      i = SQUAREFORWARD(side,sq);

      /*  Backward pawns */      
      
      if(!(bitboards.Supporters[side][sq] & my_pawns) &&  
	 (board->cboard[i] != pawn) && 
	 (bitboards.Guards[side][sq] & his_pawns)){
	  // No own pawns on left and right on lower ranks.
	  // Pawn not blocked by own or other pawn. 
	  n2=nbits(his_pawns&bitboards.Guards[side][sq]);
	  if(n2){
	      n1=nbits(my_pawns&bitboards.Neighbours[sq]);
	      if(n2>n1){
		  // More enemy pawns in front at knight's distance
		  // than own pawns on same rank.
		  //	  backward=true;
		  backward |= bitboards.BitPosArray[sq];
		  weak |= backward;
		  s += BACKWARDPAWN(board);
	      }
	  }
      

      }

      /* Pawn base under attack */
      if ((bitboards.MoveArray[ptype[side]][sq] & his_pawns) && 
	  (bitboards.MoveArray[ptype[side]][sq] & my_pawns)){
	  s += evals.pawnbaseattack;
      }
      /*  Increment file count for isolani & doubled pawn evaluation */
      nfile[FILE_(sq)]++;
   }

   for (i = 0; i <= 7; i++)
   {
      /* Doubled pawns */
       if (nfile[i] > 1){
	   s+=evals.doubled_pawn;
       }
       /* Isolated pawns */
       if (nfile[i] && (!(my_pawns & bitboards.IsolaniMask[i]))){
	   /* Isolated doubled pawns are virtually worthless */
	   doubled_pawn_cnt+=nfile[i]-1;
	   if (!(bitboards.FileBit[i] & board->b[xside][pawn])){
	       /* Isolated on a half-open file */
	       s+=evals.isolated_pawn_open+(nfile[i]-1)*evals.isolated_doubled_pawn_open;
	   }else{ 
	       /* Normal isolated pawn */
	       s += evals.isolated_pawn_normal+(nfile[i]-1)*evals.isolated_doubled_pawn_normal;
	   }
	   weak |= (my_pawns & bitboards.FileBit[i]);
      }
   }
   real_pawn_cnt=my_pawn_cnt- doubled_pawn_cnt;

    /* Penalize having eight pawns */
   if (my_pawn_cnt == 8){
        s += evals.eight_pawns;
   }

    /* Detect stonewall formation in enemy */
   if (nbits(bitboards.stonewall[xside] & board->b[xside][pawn]) == 3){
      s += evals.stonewall;
   }
    /* Locked pawns */
   //    n = 0;
   // if (side == white){
   //  n = nbits((my_pawns >> 8) & board->b[xside][pawn] &
   //           bitboards.boxes[1]);
   // }else{
   //   n = nbits((my_pawns << 8) & board->b[xside][pawn] &
   //           bitboards.boxes[1]);
   // }
   // if (n > 1){
   //	s += n * evals.lockedpawns;
   // }


   /* Save the score into the pawn hash table */ 

    pawns->passed=passed;
    pawns->weak=weak;
    pawns->backward=backward;
    pawns->attacked=attacked;
    pawns->blocked=blocked;
    pawns->score=s;
    pawns->real_pawns=real_pawn_cnt;
    pawns->phase=board->phase;
    pawns->pkey=KEY(board->PawnHashKey);

    evaldata->passed[side]=passed;
    evaldata->weak[side]=weak;
    evaldata->attacked[side]=attacked;
    evaldata->backward[side]=backward;
    evaldata->blocked[side]=blocked;
    evaldata->real_pawns[side]=real_pawn_cnt;


/***************************************************************************
 *  
 *  This section of the pawn code cannot be saved into the pawn hash as
 *  they depend on the position of other pieces.  So they have to be 
 *  calculated again.
 *
 ***************************************************************************/
phase2:

    // pawn mobility

    n = 0;
    if (side == white){
	n = nbits((my_pawns >> 8) & ~board->blocker);
    }else{
	    n = nbits((my_pawns << 8) & ~board->blocker);
    }
   	s += n * 10;


   // Pawn endgame

   if(Material[board->material_index].flags & MATERIAL_PAWN_ENDGAME){
       if(evaldata->real_pawns[side]>evaldata->real_pawns[xside]){
	   s+=evals.pawn_endgame_bonus;
       }else if(evaldata->real_pawns[side]<evaldata->real_pawns[xside]){
	   s-=evals.pawn_endgame_bonus;
       }
   }


   // pawns that can advance

   BitBoard advance;
   int move;
   int sq1;

   if(side==white){
       advance=((passed>>8) & ~board->blocker)<<8;
   }else{
       advance=((passed<<8) & ~board->blocker)>>8;
   }
   while(advance){
       sq=leadz(advance);
       CLEARBIT(advance,sq);
       sq1=SQUAREFORWARD(side,sq);
       move=MOVE(sq,sq1);
       if(SwapOffSignEx(board,move,0,false)>=0){
	   /*	   s+=60;*/
	   /*	   t_dummy=s;*/
	   s+=PHASEMIX(board,
		       evals.free_passerO[side][RANK(sq)],
		       evals.free_passerE[side][RANK(sq)]);
       }
   }



   /* Pawn on f6/c6 with Queen against castled king is very strong */
   my_pawns = board->b[side][pawn];
   sq = board->king[xside];
   /*
   if (side == white && board->b[side][queen] && 
	(bitboards.BitPosArray[C6] | bitboards.BitPosArray[F6]) & my_pawns)
   {
      if (my_pawns & bitboards.BitPosArray[F6] && sq > H6 && distance[sq][G7]==1)
	  s += evals.pawnnearking;
      if (my_pawns & bitboards.BitPosArray[C6] && sq > H6 && distance[sq][B7]==1)
	  s += evals.pawnnearking;
   }
   else if (side == black && board->b[side][queen] &&
	(bitboards.BitPosArray[C3] | bitboards.BitPosArray[F3]) & my_pawns)
   {
      if (my_pawns & bitboards.BitPosArray[F3] && sq < A3 && distance[sq][G2]==1)
	  s += evals.pawnnearking;
      if (my_pawns & bitboards.BitPosArray[C3] && sq < A3 && distance[sq][B2]==1)
	  s += evals.pawnnearking;
   }
   */

   /* Connected passed pawns on 6th or 7th rank */
   
   t = passed & bitboards.brank67[side];
   if (t && (board->pmaterial[xside] == evals.ValueR || 
	     (board->pmaterial[xside] == evals.ValueN && /*BUG: this second clause is never true as pieces is not definded */
	pieces[xside] == board->b[xside][knight])))
   {
      n1 = FILE_(board->king[xside]);
      n2 = RANK(board->king[xside]);
      for (i = 0; i <= 6; i++)
      {
	 if (t & bitboards.FileBit[i] && t & bitboards.FileBit[i+1] && (n1 < i-1 || n1 > i+1 ||
		(side == white && n2 < 4) || (side == black && n2 > 3)))
	     s += evals.connectedpp;
      }
   }

   /* Pawn on d2,e2/d7,e7 is blocked  */
   blocker = board->friends[side] | board->friends[xside];
   if (side == white && (((my_pawns & bitboards.d2e2[white]) >> 8) & blocker))
       s += evals.blockedpawn;
   if (side == black && (((my_pawns & bitboards.d2e2[black]) << 8) & blocker))
       s += evals.blockedpawn;

   /* Enemy has no pieces & King is outside of passed pawn square */
   if (passed && board->pmaterial[xside]==0)
   {
      e = board->b[xside];
      i1 = board->king[xside];
      p = passed;
      while (p)
      {
         sq = leadz (p);
	 CLEARBIT (p, sq);
	 if (board->side == side)
         {
	    if (!(bitboards.SquarePawnMask[side][sq] & board->b[xside][king]))
	       s += evals.ValueQ * Quad[side][RANK(sq)] / evals.pfactor;
         }
         else if (!(bitboards.MoveArray[king][i1] & bitboards.SquarePawnMask[side][sq]))
	    s += evals.ValueQ * Quad[side][RANK(sq)] / evals.pfactor;
      }
   }

  /* If both sides are castled on different sides, bonus for pawn storms */
  my_pawns = board->b[side][pawn];
  if (abs (FILE_ (board->king[side]) - FILE_ (board->king[xside])) >= 4 &&
           board->phase < 6)
  {
     n1 = FILE_ (board->king[xside]);
     p = (bitboards.IsolaniMask[n1] | bitboards.FileBit[n1]) & my_pawns;
     while (p)
     {
        sq = leadz (p);
        CLEARBIT (p, sq);
        s += 10 * (5 - distance[sq][board->king[xside]]);
     }
  }
  evaldata->score_p[side]=s;
   return (s);
}


static  void CTL_(board_t *board, int side, int piece, BitBoard controlled, BitBoard xcontrolled, evaldata_t *evaldata){
    int mob,cc,aek,op,eg,s;

    mob=nbits(controlled);
    cc = nbits (controlled & bitboards.boxes[0]);
    aek = nbits (xcontrolled & bitboards.DistMap[board->king[1^side]][1]);
    
    if(aek!=0){
	evaldata->score_k_attack_nb[side]++;
	evaldata->score_k_attack[side]+=evals.attackenemyking[piece]*aek;
    }

    op=evals.mobilityO[piece]*mob;
    eg=evals.mobilityE[piece]*mob;
    op+=evals.centercontrol[piece]*cc;
    eg+=evals.centercontrol[piece]*cc;
    s=PHASEMIX(board,op,eg);  
    evaldata->ctl[side][piece]+=(s+8)/16;
}

static void CTL(board_t *board, int side, evaldata_t *evaldata){
    BitBoard c,blocker,controlled,xcontrolled,blocker_noqb,blocker_noqr;
    int sq;
    blocker=board->blocker;
    blocker_noqb=blocker & ~(board->b[side][bishop]|board->b[side][queen]);
    blocker_noqr=blocker & ~(board->b[side][rook]|board->b[side][queen]);

    controlled=bitboards.MoveArray[king][board->king[side]];

    evaldata->attack.b[side][king]=controlled;
    evaldata->attack.b[side][0]=controlled;

    controlled=evaldata->attacked[side];

    evaldata->attack.b[side][pawn]=controlled;
    evaldata->attack.b[side][0]|=controlled;

    c=board->b[side][queen];
    while(c){
	sq=leadz(c);
	CLEARBIT(c,sq);
	controlled=MagicQueenAttackEx(sq,blocker);
	xcontrolled=MagicBishopAttackEx(sq,blocker_noqb);
	xcontrolled|=MagicRookAttackEx(sq,blocker_noqr);
	ASSERT(xcontrolled==AttackXFromSqBitBoard(board,sq,side));
	evaldata->attack.b[side][queen]|=controlled;
	evaldata->attack.b[side][0]|=controlled;
	CTL_(board, side, queen, controlled, xcontrolled, evaldata);
    }
    c=board->b[side][rook];
    while(c){
	sq=leadz(c);
	CLEARBIT(c,sq);
	controlled=MagicRookAttackEx(sq,blocker);
	xcontrolled=MagicRookAttackEx(sq,blocker_noqr);
	ASSERT(xcontrolled==AttackXFromSqBitBoard(board,sq,side));
	evaldata->attack.b[side][rook]|=controlled;
	evaldata->attack.b[side][0]|=controlled;
	CTL_(board, side, rook, controlled, xcontrolled, evaldata);
    }
    c=board->b[side][bishop];
    while(c){
	sq=leadz(c);
	CLEARBIT(c,sq);
	controlled=MagicBishopAttackEx(sq,blocker);
	xcontrolled=MagicBishopAttackEx(sq,blocker_noqb);
	ASSERT(xcontrolled==AttackXFromSqBitBoard(board,sq,side));
	evaldata->attack.b[side][bishop]|=controlled;
	evaldata->attack.b[side][0]|=controlled;
	CTL_(board, side, bishop, controlled, xcontrolled, evaldata);
    }
    c=board->b[side][knight];
    while(c){
	sq=leadz(c);
	CLEARBIT(c,sq);
	controlled=bitboards.MoveArray[knight][sq];
	xcontrolled=controlled;
	ASSERT(xcontrolled==AttackXFromSqBitBoard(board,sq,side));
	evaldata->attack.b[side][knight]|=controlled;
	evaldata->attack.b[side][0]|=controlled;
	CTL_(board, side, knight, controlled, xcontrolled, evaldata);
    }
}

int ScoreN (board_t *board, 
	    short side,
	    evaldata_t *evaldata)
/***************************************************************************
 *
 *  1.  central knight - distance from enemy king.
 *  2.  mobility/control/attack
 *  3.  outpost knight protected by pawn.
 *  4.  knight attacking weak pawns.
 *
 ***************************************************************************/
{
   int xside;
   int s, s1, sq;
   int EnemyKing;
   BitBoard c, t;
   
   if (board->b[side][knight] == NULLBITBOARD)
      return (0);
   xside = side^1;
   s = s1 = 0;

   c = board->b[side][knight];
   t = board->b[xside][pawn]; 
   EnemyKing = board->king[xside];

   if ( c & evaldata->pinned )
   {
	s += evals.pinnedknight * nbits(c & evaldata->pinned);
   }

   s+=evaldata->ctl[side][knight];

   while (c)
   {
      sq = leadz (c);
      CLEARBIT (c, sq);


      s1=0;

  
      if ( (bitboards.BitPosArray[sq] & bitboards.rings[3]) != NULLBITBOARD){
	s1 += evals.knightonrim;

      }

      if (MinorOutpost[side][sq] && 
	  !(t & bitboards.IsolaniMask[FILE_(sq)] & bitboards.PassedPawnMask[side][sq]) )
      {
         s1 += evals.outpostknight;

    	 /* Knight defended by own pawn */
         if (bitboards.MoveArray[ptype[xside]][sq] & board->b[side][pawn]){
            s1 += evals.outpostknight;
	 }
      }
    
      s += s1;
   }
   evaldata->score_n[side]=s;
   return (s);
}


int ScoreB (board_t *board, 
	    short side,
	    evaldata_t *evaldata)
/****************************************************************************
 *
 *  1.  double bishops.
 *  2.  mobility/control/attack
 *  3.  outpost bishop
 *  4.  fianchetto bishop
 *  5.  Bishop pair
 *
 ****************************************************************************/
{
   int xside;
   int s, s1, n, sq, EnemyKing;
   BitBoard c, t;

   if (board->b[side][bishop] == NULLBITBOARD)
      return (0);
   s = s1 = 0;
   c = board->b[side][bishop];
   xside = side ^ 1;
   EnemyKing = board->king[xside];
   n = 0;
   t = board->b[xside][pawn];

   if ( c & evaldata->pinned )
   {
	s += evals.pinnedbishop * nbits(c & evaldata->pinned);
   }

   s+=evaldata->ctl[side][bishop];

   while (c)
   {
      sq = leadz (c);
      CLEARBIT (c, sq);
      n++;

      s1=0;

      /*  Outpost bishop */
      if (MinorOutpost[side][sq] && 
	  !(t & bitboards.IsolaniMask[FILE_(sq)] & bitboards.PassedPawnMask[side][sq]))
      {
         s1 += evals.outpostbishop;

    	 /* Bishop defended by own pawn */
	 if (bitboards.MoveArray[ptype[xside]][sq] & board->b[side][pawn])
            s1 += evals.outpostbishop;
      }

      /*  Fianchetto bishop */
      if (side == white)
      {
         if (board->king[side] >= F1 && board->king[side] <= H1 && sq == G2)
	    s1 += evals.fianchetto;
         if (board->king[side] >= A1 && board->king[side] <= C1 && sq == B2)
	    s1 += evals.fianchetto;
      }
      else if (side == black)
      {
         if (board->king[side] >= F8 && board->king[side] <= H8 && sq == G7)
	    s1 += evals.fianchetto;
         if (board->king[side] >= A8 && board->king[side] <= C8 && sq == B7)
	    s1 += evals.fianchetto;
      }

      s += s1;
   }

   /* Doubled bishops */
   if (n > 1)            
       s += evals.doubledbishops;

   evaldata->score_b[side]=s;
   return (s);

}


int BishopTrapped (board_t *board, short side)
/****************************************************************************
 *
 *  Check for bishops trapped at A2/H2/A7/H7
 *
 ****************************************************************************/
{
   int s = 0;

   /* Don't waste time */
   if (board->b[side][bishop] == NULLBITBOARD)
     return (0); 

   if (side == white)
   {
      if ((board->b[white][bishop] & bitboards.BitPosArray[A7]) &&
	  (board->b[black][pawn] & bitboards.BitPosArray[B6]) && SwapOff(board,MOVE(A7,B6)) < 0)
	  s += evals.bishoptrapped;
      if ((board->b[white][bishop] & bitboards.BitPosArray[H7]) &&
	  (board->b[black][pawn] & bitboards.BitPosArray[G6]) && SwapOff(board,MOVE(H7,G6)) < 0)
	  s += evals.bishoptrapped;
   }
   else
   {
      if ((board->b[black][bishop] & bitboards.BitPosArray[A2]) &&
	  (board->b[white][pawn] & bitboards.BitPosArray[B3]) && SwapOff(board,MOVE(A2,B3)) < 0)
         s += evals.bishoptrapped;
      if ((board->b[black][bishop] & bitboards.BitPosArray[H2]) &&
	  (board->b[white][pawn] & bitboards.BitPosArray[G3]) && SwapOff(board,MOVE(H2,G3)) < 0)
         s += evals.bishoptrapped;
   }

   return (s);
}

int ScoreR (board_t *board, 
	    short side,
	    evaldata_t *evaldata)
/****************************************************************************
 *
 *  1.  rook on 7th rank and Enemy king on 8th rank or pawns on 7th rank.
 *  2.  rook on open/half-open file.
 *  3.  rook in front/behind passed pawns (pawn >= 5th rank)
 *
 ****************************************************************************/
{
   int s, s1, sq, xside, fyle, EnemyKing;
   BitBoard c;

   if (board->b[side][rook] == NULLBITBOARD)
      return (0);
   s = s1 = 0;
   c = board->b[side][rook];
   xside = side ^ 1;
   EnemyKing = board->king[xside];

   if ( c & evaldata->pinned )
   {
	s += evals.pinnedrook * nbits(c & evaldata->pinned);
   }

   s+=evaldata->ctl[side][rook];
   while (c)
   {
      sq = leadz (c);
      CLEARBIT (c, sq);

      s1=0;

      fyle = FILE_(sq);
      if (board->phase < 7)
      {
         if (!(board->b[side][pawn] & bitboards.FileBit[fyle]))
         {
	    if (fyle == 5 && FILE_(board->king[xside])>=E_FILE)
	      s1 += evals.rookliberated;
            s1 += evals.rookhalffile;
            if (!(board->b[xside][pawn] & bitboards.FileBit[fyle]))
		s1 += evals.rookopenfile;
         }
      }

      if (board->phase > 6 && (bitboards.FileBit[fyle] & evaldata->passed[white] & bitboards.boardhalf[black]))
      {
	 if (nbits (bitboards.Ray[sq][7] & evaldata->passed[white]) == 1)
	    s1 += evals.rookbehindpp;
	 else if (bitboards.Ray[sq][4] & evaldata->passed[white])
            s1 += evals.rookinfrontpp;
      }
      if (bitboards.FileBit[fyle] & evaldata->passed[black] & bitboards.boardhalf[white])
      {
	 if (nbits (bitboards.Ray[sq][4] & evaldata->passed[black]) == 1)
	     s1 += evals.rookbehindpp;
	 else if (bitboards.Ray[sq][7] & evaldata->passed[black])
	     s1 += evals.rookinfrontpp;
      }


      /* Rook on 7th rank */
      if (RANK (sq) == rank7[side] && (RANK (EnemyKing) == rank8[side] ||
	  board->b[xside][pawn] & bitboards.RankBit[RANK(sq)]))
         s1 += evals.rook7rank;

      s += s1;
   }
   evaldata->score_r[side]=s;
   return (s);
}

int DoubleQR7 (board_t *board, short side)
/***************************************************************************
 *
 *  This code just check to see if there is a QQ or QR or RR combo on the
 *  7th rank.  This is very strong and given quite a big bonus.  This 
 *  routine is called by the lazy section.
 *
 ***************************************************************************/
{
   int xside;

   xside = 1^side;
   if (nbits ((board->b[side][queen]|board->b[side][rook]) & bitboards.brank7[side]) > 1
      && ((board->b[xside][king] & bitboards.brank8[side]) || 
	  (board->b[xside][pawn] & bitboards.brank7[side])))

       return evals.rooks7rank;
   else
      return (0);
}

int ScoreQ (board_t *board, 
	    short side,
	    evaldata_t *evaldata)
/***************************************************************************
 *
 *  1. queen centralization.
 *  2. king tropism.
 *  3. Bonus if opponent king is exposed.
 *
 ***************************************************************************/
{
   int xside;
   int s, s1, sq, EnemyKing;
   BitBoard c;
   
   s = s1 = 0;

   /* Try to keep our queen on the board for attacking purposes. */
   if (board->b[side][queen] == NULLBITBOARD) {
       s += evals.queen_not_present;
       return(s);
    }                                                                           

   xside = 1 ^ side;
   c = board->b[side][queen];
   EnemyKing = board->king[xside];

   if ( c & evaldata->pinned )
   {
       s += evals.pinnedqueen * nbits(c & evaldata->pinned);
   }

   s+=evaldata->ctl[side][queen];
   while (c)
   {
      sq = leadz (c);
      CLEARBIT (c, sq);

      s1=0;
      /*
      if (distance[sq][EnemyKing] <= 2)
	  s1 += evals.queennearking;
      */

      s += s1;
   }
   evaldata->score_q[side]=s;
   return (s);
}


int ScoreK(board_t *board,
	    short side,
	    evaldata_t *evaldata){
    int ks_,xside;
    xside=1^side;
    if(board->b[1^side][queen]==NULLBITBOARD){
	return evaldata->score_k[side];
    }
    ASSERT(evaldata->score_k[side]==evaldata->score_k_1[side]);
    ks_=-(evaldata->score_k_attack[xside]*KingAttackWeight[evaldata->score_k_attack_nb[xside]])/256;
    evaldata->score_ks[side]=ks_;
    evaldata->score_k[side]+=ks_;
    ASSERT(evaldata->score_k[side]==evaldata->score_k_1[side]+evaldata->score_ks[side]);
    return evaldata->score_k[side];
    
}

int ScoreK1 (board_t *board, 
	    short side,
	    evaldata_t *evaldata)
/***************************************************************************
 *
 *  1.  king in the corner. ?? SRW 2002-08-02 Unclear if implemented
 *  2.  pawns around king.
 *  3.  king on open file.
 *  4.  Uncastled king.
 *  5.  Open rook file via Ng5 or Bxh7 sac.
 *  6.  No major or minor piece in the king's quadrant.
 *
 ***************************************************************************/
{
   int xside;
   int s, sq, sq1, n, n1, n2, file, fsq, rank;
   BitBoard b, x;
   short KingSafety[2];

   
   s = 0;
   xside = 1^side;
   sq = board->king[side];
   file = FILE_ (sq);
   rank = RANK (sq);
   KingSafety[side] = 0;
   if (!ENDING(board))
   { 

       //      s += ((6 - board->phase) * KingSq[sq] + board->phase * EndingKing[sq]) / 6;

      /* After castling kingside, reward having all 3 pawns in front but not if
	 there is a threatening pawn. This permits the freeing move F3/F6. */
	 
      //      if (side == white)
      //  n = nbits (bitboards.MoveArray[king][sq] & board->b[side][pawn] & bitboards.RankBit[rank+1]);
      // else
      //  n = nbits (bitboards.MoveArray[king][sq] & board->b[side][pawn] & bitboards.RankBit[rank-1]);
      // This is simpler!
      n = nbits(bitboards.PawnCover[side][sq] & board->b[side][pawn]);
      s += pawncover[n];

      /* Penalize compromised wing pawn formations prior to castle. */
      // This doesn't seem to have any positive effect.
      //      if (!board->castled[side]) {
      //  n = -1;
      //  if (side == white) {
      //    /* King on original square and unmoved */
      //	  if (sq == 4 && board->Mvboard[sq] == 0) {
      //	    /* Kingside - rook on original square and unmoved. */
      //	    if ( (board->b[side][rook] & bitboards.BitPosArray[H1])!=NULLBITBOARD &&
      //		 board->Mvboard[H1] == 0)
      //             n = nbits (bitboards.MoveArray[king][G1] & 
      //			      board->b[side][pawn] & bitboards.RankBit[rank+1]);
      //	    /* Queenside */
      //	    if ( (board->b[side][rook] & bitboards.BitPosArray[A1])!=NULLBITBOARD &&
      //		 board->Mvboard[A1] == 0)
      //            n = nbits (bitboards.MoveArray[king][C1] & 
      //			      board->b[side][pawn] & bitboards.RankBit[rank+1]);
      //   }
      //	} else {
      //	  if (sq == 60 && board->Mvboard[sq] == 0) {
      //	    /* Kingside */
      //	    if ( (board->b[side][rook] & bitboards.BitPosArray[H8])!=NULLBITBOARD &&
      //		 board->Mvboard[H8] == 0)
      //	                 n = nbits (bitboards.MoveArray[king][G8] & 
      //			      board->b[side][pawn] & bitboards.RankBit[rank-1]);
      //	    /* Queenside */
      //	    if ( (board->b[side][rook] & bitboards.BitPosArray[A8])!=NULLBITBOARD &&
      //		 board->Mvboard[A8] == 0)
      //            n = nbits (bitboards.MoveArray[king][C8] & 
      //		      board->b[side][pawn] & bitboards.RankBit[rank-1]);
      //  }
      //	}
        
      //	/* Penalize breaking the wing pawn formations prior to castle */
      //	if (n != -1) s += pawncover[n];
      //}
   
      /*   if (file >= F_FILE && 
		!(bitboards.FileBit[G_FILE] & board->b[side][pawn])){
	  if (side == white && board->cboard[F2] == pawn){
	      s += evals.gopen;
	  }else if (side == black && board->cboard[F7] == pawn){
	      s += evals.gopen;
	  }
	  }*/

      /* No friendly pawns on this king file */
      /*      if (!(bitboards.FileBit[file] & board->b[side][pawn]))
	      s += evals.kingopenfile;*/

      /* No enemy pawns on this king file */
      /*      if (!(bitboards.FileBit[file] & board->b[xside][pawn]))
	      s += evals.kingopenfile1;*/

      /*      switch (file)
      {
         case A_FILE :
         case E_FILE :
         case F_FILE :
         case G_FILE : if (!(bitboards.FileBit[file+1] & board->b[side][pawn]))
	      s += evals.kingopenfile;
	     if (!(bitboards.FileBit[file+1] & board->b[xside][pawn]))
		 s += evals.kingopenfile1;
	     break;
         case H_FILE :
         case D_FILE :
         case C_FILE :
	 case B_FILE : if (!(bitboards.FileBit[file-1] & board->b[side][pawn]))
		          s += evals.kingopenfile;
		       if (!(bitboards.FileBit[file-1] & board->b[xside][pawn]))
		          s += evals.kingopenfile1;
	               break;
	 default :
	               break;
		       }*/

      if (board->castled[side]) {
        if (side == white) {
          if (file > E_FILE) {
            if (!(bitboards.BitPosArray[F2] & board->b[side][pawn]) ||
                !(bitboards.BitPosArray[G2] & board->b[side][pawn]) ||
                !(bitboards.BitPosArray[H2] & board->b[side][pawn]) )
                s += evals.rupture;
          } else if (file < E_FILE) {
            if (!(bitboards.BitPosArray[A2] & board->b[side][pawn]) ||
                !(bitboards.BitPosArray[B2] & board->b[side][pawn]) ||
                !(bitboards.BitPosArray[C2] & board->b[side][pawn]) )
                s += evals.rupture;
          }
        } else {
          if (file > E_FILE) {
            if (!(bitboards.BitPosArray[F7] & board->b[side][pawn]) ||
                !(bitboards.BitPosArray[G7] & board->b[side][pawn]) ||
                !(bitboards.BitPosArray[H7] & board->b[side][pawn]) )
                s += evals.rupture;
          } else if (file < E_FILE) {
            if (!(bitboards.BitPosArray[A7] & board->b[side][pawn]) ||
                !(bitboards.BitPosArray[B7] & board->b[side][pawn]) ||
                !(bitboards.BitPosArray[C7] & board->b[side][pawn]) )
                s += evals.rupture;
	  }
	}
      }

	/* Stock piece sacrifice on h file */

      /*	if (file >= E_FILE && board->b[xside][queen] && board->b[xside][rook] &&
	    !((board->b[side][pawn]|board->b[xside][pawn]) & bitboards.FileBit[7])){
         s += evals.hopen;
	 }*/
	/* King trapping rook */
        if (side == white) {
	  if (file > E_FILE) {
	    if (board->b[side][rook]&bitboards.mask_kr_trapped_w[H_FILE-file]) {
		s += evals.rooktrapped;
	    }
	  } else if (file < D_FILE) {
	    if (board->b[side][rook]&bitboards.mask_qr_trapped_w[file]) {
		s += evals.rooktrapped;
	    }
	  }
	} else {
	  if (file > E_FILE) {
	    if (board->b[side][rook]&bitboards.mask_kr_trapped_b[H_FILE-file]) {
		s += evals.rooktrapped;
	    }
	  } else if (file < D_FILE) {
	    if (board->b[side][rook]&bitboards.mask_qr_trapped_b[file]) {
		s += evals.rooktrapped;
	    }
	  }
	}

      /* Don't give fianchetto target for advanced pawns */
	/*      if (file > E_FILE && FILE_(board->king[xside]) < D_FILE) {
         if (side == white) fsq = G3; else fsq = G6;
	 if ((bitboards.BitPosArray[fsq] & board->b[side][pawn]) != NULLBITBOARD)
            if (((bitboards.BitPosArray[F4]|bitboards.BitPosArray[H4]|
	         bitboards.BitPosArray[F5]|bitboards.BitPosArray[H5])
	      & board->b[xside][pawn]) != NULLBITBOARD) 
	        s += evals.fianchetto_target;	
		}*/
	/*      if (file < E_FILE && FILE_(board->king[xside]) > E_FILE) {
         if (side == white) fsq = B3; else fsq = B6;
	 if ((bitboards.BitPosArray[fsq] & board->b[side][pawn]) != NULLBITBOARD)
            if (((bitboards.BitPosArray[A4]|bitboards.BitPosArray[C4]|
	         bitboards.BitPosArray[A5]|bitboards.BitPosArray[C5])
	      & board->b[xside][pawn]) != NULLBITBOARD) 
	        s += evals.fianchetto_target;
		}*/

	/* No major/minor piece in king's quadrant */
	/* First identify the quadrant */
	x = bitboards.boardhalf[side] & bitboards.boardside[file<=D_FILE];
	/* Now identify the number of non-pawn enemy in quadrant */
        n1 = nbits(x & (board->friends[xside]));
        /* Now identify the number of non-pawn friends in quadrant */
	if (n1 > 0) {
	    n2 = nbits(x & (board->friends[side] & ~board->b[side][pawn] & 
			    ~board->b[side][king]));
	    if (n1 > n2)
		s += (n1 - n2) * evals.king_defender_deficit;
	}
	
	KingSafety[side] = s;
	s = (s * factor[board->phase]) / 8;
   }
   else
   {
       //      s += EndingKing[sq];
      //  s += CTL(board, sq,king,side,evaldata);
      b = (board->b[white][pawn] | board->b[black][pawn]);
      while (b)
      {
	  sq1 = leadz (b);
	  CLEARBIT (b, sq1);
	  if (bitboards.BitPosArray[sq1] & board->b[white][pawn]){
	      s -= dist_penalty[distance[sq][sq1+8]];
	      if(bitboards.BitPosArray[sq1] & evaldata->passed[white]){
		  s -= distance[sq][sq1+8] * 3;
	      }
	  }else if (bitboards.BitPosArray[sq1] & board->b[black][pawn]){
	      s -= dist_penalty[distance[sq][sq1-8]];
	      if(bitboards.BitPosArray[sq1] & evaldata->passed[black]){
		  s -= distance[sq][sq1-8] * 3;
	      }
	  }
      }


   }

    if (board->phase >= 4) {
      /* Weak back rank */
      if (side == white) {
        if (sq < A2) 
	  if (!(bitboards.MoveArray[king][sq] & (~board->b[side][pawn] & bitboards.RankBit[1]))) 
	      s += evals.king_back_rank_weak;
      } else {
	if (sq > H7) 
	  if (!(bitboards.MoveArray[king][sq] & (~board->b[side][pawn] & bitboards.RankBit[6]))) 
	    s += evals.king_back_rank_weak;
      }
   }
    evaldata->score_k_1[side]=s;
    evaldata->score_k[side]=s;
    return (s);
}


int LoneKing (board_t *board, int side, int loser)
/**************************************************************************
 *
 *  One side has a lonely king and the other has no pawns, but enough
 *  mating material.  We give an additional bonus of 150 points for the 
 *  winning side to attract the search to such positions.
 *
 **************************************************************************/
{
   // We must have a hard look at this routine..

   int s, winner, sq1, sq2;

   winner = 1^loser;
   if (board->material[winner] == evals.ValueB+evals.ValueN && 
	nbits(board->b[winner][bishop]) == 1 &&
       nbits(board->b[winner][knight]) == 1){
     return (ScoreKBNK (board, side, loser));
   }

   
   if(board->material[winner]<evals.ValueR){
       return DRAWSCORE;
   }

   if(nbits(board->b[winner][knight])==2 && board->material[winner]==2*evals.ValueN){
       return DRAWSCORE;
   }

   sq1 = board->king[winner];
   sq2 = board->king[loser];
   s = 150 - 6 * taxicab[sq1][sq2] - EndingKing[sq2];
   if (side == loser)
      s = -s;
   s += MATERIALE(board,side); 

   return (s);
}


int KBNK[64] = 
{
   0, 10, 20, 30, 40, 50, 60, 70,
  10, 20, 30, 40, 50, 60, 70, 60,
  20, 30, 40, 50, 60, 70, 60, 50,
  30, 40, 50, 60, 70, 60, 50, 40,
  40, 50, 60, 70, 60, 50, 40, 30,
  50, 60, 70, 60, 50, 40, 30, 20,
  60, 70, 60, 50, 40, 30, 20, 10,
  70, 60, 50, 40, 30, 20, 10,  0
};

int ScoreKBNK (board_t *board, int side, int loser)
/****************************************************************************
 *
 *  My very own KBNK routine!
 *
 ****************************************************************************/
{
   int s, winner, sq1, sq2, sqB;

   winner = 1^loser;
   sqB = board->king[loser];
   if (board->b[winner][bishop] & bitboards.WhiteSquares)
      sqB = RANK(sqB)*8 + 7 - FILE_(sqB);
   sq1 = board->king[winner];
   sq2 = board->king[loser];
   s = 300 - 6 * taxicab[sq1][sq2];
   s -= KBNK[sqB];
   s -= EndingKing[sq2];
   s -= taxicab[leadz(board->b[winner][knight])][sq2];
   s -= taxicab[leadz(board->b[winner][bishop])][sq2];

   /*  King in the central 4x4 region is good! */
   if (board->b[winner][king] & bitboards.rings[0])
      s += 20;
   if (side == loser)
      s = -s;
   s += MATERIALE(board,side); 

   return (s); 
}


int ScoreDev (board_t *board, short side, evaldata_t *evaldata)
/***************************************************************************
 *
 *  Calculate the development score for side (for opening only).
 *  Penalize the following.
 *  .  Uncastled and cannot castled
 *  .  Undeveloped knights and bishops
 *  .  Early queen move.
 *
 ***************************************************************************/
{
   int s;
   int sq;
   BitBoard c;

   /* Calculate whether we are developed */
   c = (board->b[side][knight] & bitboards.nn[side]) | (board->b[side][bishop] & bitboards.bb[side]);
   s = nbits(c) * evals.undevelopedminor;

   /* If we are castled or beyond the 20th move, no more ScoreDev */
   if (board->castled[side] || board->GameCnt >= 38){
       evaldata->score_dev[side]=s;
       return (s);
   }

   s += evals.notcastled;

   /* If the king is moved, nail it, otherwise check rooks */
   if (board->Mvboard[board->king[side]] > 0) 
      s += evals.kingmoved;

   /* Discourage rook moves */
   c = board->b[side][rook];
   while (c) {
     sq = leadz(c);
     CLEARBIT(c, sq);
     if (board->Mvboard[sq] > 0)
       s += evals.rookmoved;
   }

   /* Penalize a queen that moves at all */
   if (board->b[side][queen])
   {
      sq = leadz (board->b[side][queen]);
      if (board->Mvboard[sq] > 0)
         s += evals.earlyqueenmove;
         /* s += Mvboard[sq] * EARLYQUEENMOVE; */
   }

   /* Discourage repeat minor piece moves */
   c = board->b[side][knight] | board->b[side][bishop];
   while (c) {
     sq = leadz(c);
     CLEARBIT(c, sq);
     if (board->Mvboard[sq] > 1)
	 s += evals.earlyminorrepeat;;
	/* s += Mvboard[sq] * EARLYMINORREPEAT; */
   }

   /* Discourage any wing pawn moves */
/*   c = board->b[side][pawn] & (FileBit[0]|FileBit[1]|FileBit[6]|FileBit[7]); */
   c = board->b[side][pawn] & bitboards.bfile0167;
   while (c) {
     sq = leadz(c);
     CLEARBIT(c, sq);
     if (board->Mvboard[sq] > 0) 
	 s += evals.earlywingpawnmove;
   }

   /* Discourage any repeat center pawn moves */
/*   c = board->b[side][pawn] & (FileBit[2]|FileBit[3]|FileBit[4]|FileBit[5]); */
   c = board->b[side][pawn] & bitboards.bfile2345;
   while (c) {
     sq = leadz(c);
     CLEARBIT(c, sq);
     if (board->Mvboard[sq] > 1) 
	s += evals.earlycenterrepeat;
   }
   evaldata->score_dev[side]=s;
   return (s);
}


/*  Array of pointer to functions  */
static int (*ScorePiece[7]) (board_t *, short, evaldata_t *) =
{ NULL, ScoreP, ScoreN, ScoreB, ScoreR, ScoreQ, ScoreK };



int Evaluate (board_t *board, 
	      searchdata_t *searchdata, 
	      searchstats_t *searchstats, 
	      evaldata_t *evaldata,
	      int alpha, 
	      int beta)
/****************************************************************************
 *
 *  First check to see if this position can be specially dealt with.
 *  E.g. if our bounds indicate that we are looking for a mate score,
 *  then just return the material score.  Nothing else is important.
 *  If its a KPK endgame, call our KPK routine.
 *  If one side has a lone king & the winning side has no pawns then call
 *  the LoneKing() mating driver routine.  Note that there is enough
 *  mating material as we have already check for insufficient mating material
 *  in the call to EvaluateDraw() in search()/quiesce().
 *
 ****************************************************************************/
{
   int side, xside;
   int score;
   int piece;
   BitBoard *b;
   int ending, recog;
   uint32_t EG_flags;
   int EG_score=0;
   int no_lazy_eval=false;
   int side_has_no_mating_material;
   int xside_has_no_mating_material;
   int side_has_no_pawns;
   int xside_has_no_pawns;
   int side_can_sac;
   int xside_can_sac;
   attack_t attack[1];
   BitBoard pinned;
   BitBoard pieces[2];
   int mat_flags;
   int score_k_1;
   etable_slot_t *p;
   int flags=0;

   //   For testing!
   //   return MATERIALX(board,board->side);


   // for safety (and for speed we do not call EvalDataInit....)!
   memset(evaldata,0,sizeof(evaldata_t));

   p=ETableGet(ETable, board);
   if(EVAL_VALID(board,p)){
       //       return p->score;
       switch(p->flags){
       case ETABLE_LOWERBOUND:
	   if(p->score>=beta){
	       return p->score;
	   }
	   break;
       case ETABLE_UPPERBOUND:
	   if(p->score<=alpha){
	       return p->score;
	   }
	   break;
       case ETABLE_EXACTSCORE:
	   return p->score;
	   break;
       }
   }

     if(beta!=alpha+1){
         alpha=-INFIN;
          beta=INFIN;
      }

   EG_flags=EG_NOTRECOGNIZED;
   searchstats->EvalCnt++;
   ending=ENDING(board);
   
   recog=Material[board->material_index].recog;
   mat_flags=Material[board->material_index].flags;

   side = board->side;
   xside = 1 ^ side;

   /*  If we are looking for a MATE, just return the material */
   if (alpha > MATE-255 || beta < -MATE+255){
       score=(MATERIALO(board,side));  /* Faster than MATERIALX */
       flags=ETABLE_EXACTSCORE;
       goto finish;
   }

   /*  A KPK endgame. */
   if (recog==MATERIAL_KPK){
       EG_score=ScoreKPK (board, &EG_flags);
       if(EG_flags==EG_EXACTSCORE){ // always true
	   score=EG_score;
	   flags=ETABLE_EXACTSCORE;
	   goto finish;
       }
   }

   if (recog==MATERIAL_KPPP_K){
       EG_score=ScoreKPPP_K (board, &EG_flags);
       if(EG_flags==EG_EXACTSCORE){
	   score=EG_score;
	   flags=ETABLE_EXACTSCORE;
	   goto finish;
       }
   }

   if (recog==MATERIAL_KPPP_KPP_){
       EG_score=ScoreKPPP_KPP_ (board, &EG_flags);
       if(EG_flags==EG_EXACTSCORE){ // this does not happen now
	   score=EG_score;
	   flags=ETABLE_EXACTSCORE;
	   goto finish;
       }
   }

   if(EG_flags!=EG_NOTRECOGNIZED){
       no_lazy_eval=true;
   }

   /*  One side has a lone king and other side has no pawns */
   if (ending && board->material[xside] == 0 && board->b[side][pawn] == NULLBITBOARD){
     score=LoneKing (board, side, xside);
     flags=ETABLE_EXACTSCORE;
     goto finish;
   }
   if (ending && board->material[side] == 0 && board->b[xside][pawn] == NULLBITBOARD){
     score=LoneKing (board, side, side);
     flags=ETABLE_EXACTSCORE;
     goto finish;
   }

   if(recog==MATERIAL_KBPK){
       no_lazy_eval=true;
       EG_score=ScoreKBPK(board,&EG_flags);
       if(EG_flags==EG_EXACTSCORE){
	   score=EG_score;
	   flags=ETABLE_EXACTSCORE;
	   goto finish;
       }
   }

   if(recog==MATERIAL_KRPKR){
       no_lazy_eval=true;
       EG_score=ScoreKRPKR(board,&EG_flags);
       if(EG_flags==EG_EXACTSCORE){
	   score=EG_score;
	   flags=ETABLE_EXACTSCORE;
	   goto finish;
       }
   }

   if(recog==MATERIAL_KRKP){
       no_lazy_eval=true;
       EG_score=ScoreKRKP(board,&EG_flags);
       if(EG_flags==EG_EXACTSCORE){
	   score=EG_score;
	   flags=ETABLE_EXACTSCORE;
	   goto finish;
       }
   }

   if(side==white){
       side_has_no_mating_material=mat_flags & MATERIAL_WHITE_NO_MATING;
       xside_has_no_mating_material=mat_flags & MATERIAL_BLACK_NO_MATING;
   }else{
       side_has_no_mating_material=mat_flags & MATERIAL_BLACK_NO_MATING;
       xside_has_no_mating_material=mat_flags & MATERIAL_WHITE_NO_MATING;
   }

   if(side==white){
       side_can_sac = mat_flags & MATERIAL_WHITE_CAN_SAC;
       xside_can_sac =mat_flags & MATERIAL_BLACK_CAN_SAC;
   }else{
       side_can_sac = mat_flags & MATERIAL_BLACK_CAN_SAC;
       xside_can_sac =mat_flags & MATERIAL_WHITE_CAN_SAC;
   }

	   
   side_has_no_pawns=ending && !board->b[side][pawn]; 
   
   xside_has_no_pawns=ending &&	!board->b[xside][pawn]; 

   if(ending && (side_has_no_mating_material || 
      xside_has_no_mating_material || 
      side_has_no_pawns || 
		 xside_has_no_pawns)){
       no_lazy_eval=true;
   }

/****************************************************************************
 *
 *  Lets try a lazy evaluation.  In this stage, we should evaluate all those
 *  features that gives big bonus/penalties.  E.g. squares around king is
 *  attacked by enemy pieces, 2 rooks on 7th rank, runaway passed pawns etc.
 *  This will be the direction, so things will continue to change in this
 *  section.
 *
 ****************************************************************************/
   int lazy_material, lazy_eval, lazy_pos, slow_pos;
   
   if(search.RandomizeEval){
       lazy_material=RANDOM(board->HashKey) & 0x1;
   }else{
       lazy_material=0;
   }
   lazy_material+=MATERIALX(board,side);
   lazy_material+=PHASEMIX(board,evals.stm_bonusO,evals.stm_bonusE);

   if (!no_lazy_eval && lazy_material + searchdata->posdiff[side]/*+150*/ <= alpha 
        &&  board->phase <= 6 )   {
       searchstats->EvalMatCnt++;
       score=lazy_material+searchdata->posdiff[side] ;
       flags=ETABLE_UPPERBOUND;
       goto finish;
    }
   if (!no_lazy_eval && lazy_material - searchdata->posdiff[xside]/* -150*/>= beta 
        &&  board->phase <= 6 )   {
       searchstats->EvalMatCnt++;
       score=lazy_material-searchdata->posdiff[xside];
       flags=ETABLE_LOWERBOUND;
       goto finish;
    }
   lazy_pos = 0;


   lazy_pos  += ScoreDev (board, side,evaldata) - ScoreDev (board, xside,evaldata);
   lazy_pos += ScoreP (board, side, evaldata) 
       - ScoreP (board, xside, evaldata);
   lazy_pos += (score_k_1=(ScoreK1 (board, side, evaldata) 
			   - ScoreK1 (board, xside, evaldata)));
   lazy_pos += BishopTrapped (board,side) - BishopTrapped (board,xside);
   lazy_pos += DoubleQR7 (board, side) - DoubleQR7 (board, xside);
/***************************************************************************
 *
 *  Add adjustments from material table
 *
 ***************************************************************************/
   if(side==white){
       lazy_pos+=Material[board->material_index].score;
   }else{
       lazy_pos-=Material[board->material_index].score;
   }

   lazy_eval=lazy_pos + lazy_material;

/**************************************************************************
 *
 *  See if we can have a lazy evaluation cut.  Otherwise its a slow eval.
 * 
 **************************************************************************/

   if (!no_lazy_eval && lazy_eval + searchdata->lazydiff[side] /*+50*/ <= alpha)
   {
       searchstats->EvalLazyCnt++;
       score=lazy_eval + searchdata->lazydiff[side];
       flags=ETABLE_UPPERBOUND;
       goto finish;
   }
   else if (!no_lazy_eval && lazy_eval - searchdata->lazydiff[xside] /*-50*/ >= beta)
   {
       searchstats->EvalLazyCnt++;
       score=lazy_eval- searchdata->lazydiff[xside] ;
       flags=ETABLE_LOWERBOUND;
       goto finish;
   }

   searchstats->EvalFullCnt++;

   slow_pos=0;
		
   slow_pos-=score_k_1;
   CTL(board, side, evaldata);
   CTL(board, xside, evaldata);

   evaldata->pinned=PinsFind(board, &(evaldata->attack));

   for (piece = knight; piece <= king; piece++) {
       slow_pos += (*ScorePiece[piece]) (board, side, evaldata) 
	   - (*ScorePiece[piece]) (board, xside, evaldata);
   }
   score = lazy_eval + slow_pos;


/***************************************************************************
 *
 *  Opposite color bishops is drawish.
 *
 ***************************************************************************/

   b = board->b[white];
   pieces[white] = b[knight] | b[bishop] | b[rook] | b[queen];
   b = board->b[black];
   pieces[black] = b[knight] | b[bishop] | b[rook] | b[queen];


   if (/* EG_flags==EG_NOTRECOGNIZED && ending && */ pieces[white] == board->b[white][bishop] && 
       pieces[black] == board->b[black][bishop] &&
       ((pieces[white] & bitboards.WhiteSquares && pieces[black] & bitboards.BlackSquares) ||
	(pieces[white] & bitboards.BlackSquares && pieces[black] & bitboards.WhiteSquares))) {
         score /= 2;
       //       EG_flags=EG_MULTIPLIER;
       //       EG_score=8;
   }

    
/***************************************************************************
 *
 *  When one side has no mating material, then his score can never be > 0.
 *
 ***************************************************************************/

   if (/* EG_flags==EG_NOTRECOGNIZED && */ score > 0 && side_has_no_mating_material){
             score = 0;
	     //EG_flags=EG_UPPERBOUND;
	     //EG_score=0;
   }
   if (/* EG_flags==EG_NOTRECOGNIZED && */ score < 0 && xside_has_no_mating_material){
          score = 0;
	  // EG_flags=EG_LOWERBOUND;
	  //EG_score=0;
   }

/**************************************************************************
 *
 * See if we can sacrifice a minor for a pawn.
 *
 *************************************************************************/

   if(/* EG_flags==EG_NOTRECOGNIZED && */ score<0 && side_can_sac){
              score/=2;
	      //EG_flags=EG_MULTIPLIER;
	      //EG_score=8;
   }

   if(/* EG_flags==EG_NOTRECOGNIZED && */ score>0 && xside_can_sac){
              score/=2;
	      //EG_flags=EG_MULTIPLIER;
	      //EG_score=8;
   }

/***************************************************************************
 *
 *  If we have no pawns then we need more than the value of a Bishop extra to win
 *  In some cases this is not correct (fix this).
 *
 ***************************************************************************/

   if (/* EG_flags==EG_NOTRECOGNIZED && */ score > 0 && side_has_no_pawns 
       && (MATERIAL(board,side)+nbits(board->b[xside][pawn])*evals.ValueP<=evals.ValueB)){
              score = MIN(score,48-EndingKing[board->king[xside]]); 
       //       EG_flags=EG_UPPERBOUND;
       //       EG_score=48-EndingKing[board->king[xside]]; 
   }
   
   if (/* EG_flags==EG_NOTRECOGNIZED && */ score < 0 && xside_has_no_pawns
       && (MATERIAL(board,xside)+nbits(board->b[side][pawn])*evals.ValueP<=evals.ValueB)){
              score = MAX(score,-48+EndingKing[board->king[side]]); 
	      //EG_flags=EG_LOWERBOUND;
	      //EG_score=-48+EndingKing[board->king[side]];
   }

   if(ending){
       switch(EG_flags){
       case EG_LOWERBOUND:
	   score=MAX(score,EG_score);
	   break;
       case EG_UPPERBOUND:
	   score=MIN(score,EG_score);
	   break;
       case EG_MULTIPLIER:
	   score=(score*EG_score)>>4;
	   break;
       }
   }

   //   if(ending && EG_flags==EG_LOWERBOUND){
   //       score=MAX(score,EG_score);
   //   }
   //   if(ending && EG_flags==EG_UPPERBOUND){
   //       score=MIN(score,EG_score);
   //   }



   slow_pos=score-lazy_eval;
   
   searchdata->posdiff[side]=MAX(searchdata->posdiff[side],lazy_pos+slow_pos);
   searchdata->lazydiff[side]=MAX(searchdata->lazydiff[side],slow_pos);
   searchdata->posdiff[xside]=MAX(searchdata->posdiff[xside],-lazy_pos-slow_pos);
   searchdata->lazydiff[xside]=MAX(searchdata->lazydiff[xside],-slow_pos);

   flags=ETABLE_EXACTSCORE;

 finish:
   p->flags=flags;
   p->score=score;
   p->ekey=KEY(board->HashKey);
   return (score);
}






short EvaluateDraw (board_t *board){
/***************************************************************************
 *
 * This is an interior draw recognizer. It should only return draw if
 * this is absolutely certain. Currently this is not entirely true.
 * TODO: fix this.
 *
 ***************************************************************************/

    BitBoard *w, *b;
    int wm, bm;
    uint32_t EG_flags;
    int recog;

    recog=Material[board->material_index].recog;
   

    if(!ENDING(board)){
	return  false;
    }


    w = board->b[white];
    b = board->b[black];


    if(recog==MATERIAL_KPK && 
       (ScoreKPK(board,&EG_flags)==DRAWSCORE) && 
       (EG_flags==EG_EXACTSCORE)){
        return true;
    }


    if(recog==MATERIAL_KBPK && 
       (ScoreKBPK(board,&EG_flags)==DRAWSCORE) && 
       (EG_flags==EG_EXACTSCORE)){
	return true;
    }
    
    if (w[pawn] != 0 || b[pawn] != 0){
	return false;
    }

    wm = board->material[white];
    bm = board->material[black];

    if(wm+bm < evals.ValueR){
	return true;
    }

    // Some "draw" endgames are mate in 1 if the king is in the corner.
    if((w[king]|b[king]) & bitboards.Corners){
	return false;
    }
    if((wm<evals.ValueR) && (bm<evals.ValueR)){
	return true;
    }
    // These are not yet entirely correct. See
    // http://talkchess.com/forum/viewtopic.php?t=40446&postdays=0&
    // postorder=asc&topic_view=&start=10 .
    if(wm==2*evals.ValueN && nbits(w[knight])==2 && bm==0){
	return true;
    }
    if(bm==2*evals.ValueN && nbits(b[knight])==2 && wm==0){
	return true;
    }
    // Perhaps we should drop this. It is very rare.
    if (wm < evals.ValueR){
	if (bm == 2*evals.ValueB && 
	    ( nbits(board->b[black][bishop] & bitboards.WhiteSquares) == 2 ||
	      nbits(board->b[black][bishop] & bitboards.BlackSquares) == 2 )){
	    return true;
	}
    }
    if (bm < evals.ValueR){
	if (wm == 2*evals.ValueB && 
	    ( nbits(board->b[white][bishop] & bitboards.WhiteSquares) == 2 ||
	      nbits(board->b[white][bishop] & bitboards.BlackSquares) == 2 )){
	    return true;
	}
    }
    return false;
}

