#include <stdio.h>


#include "config.h"
#include "endgame.h"
#include "move.h"
#include "log.h"
#include "genmove.h"


#include "gtb.h"

gtb_t GTB[1];

#if !defined(HAVE_GTB_PROBE_H) || !defined(HAVE_LIBGTB)

void GTBInit(gtb_t *gtb){
    gtb->have_libgtb=false;
    gtb->tbhits=0;
}

void GTBSync(gtb_t *gtb){
    return;
}

void GTBSetPath(gtb_t *gtb, char *path){
    return;
}

void GTBSetCacheSize(gtb_t *gtb, size_t cache_size){
    return;
}

void GTBSetWDLFraction(gtb_t *gtb, int wdl_fraction){
    return;
}

void GTBSetScheme(gtb_t *gtb, int scheme){
    return;
}

void GTBSetVerbosity(gtb_t *gtb, int verbosity){
    return;
}

int GTBScore(gtb_t *gtb, board_t *board, int mode, int *flags){
    *flags=EG_NOTRECOGNIZED;
    return EG_SCORENOTRECOGNIZED;

}

void GTBQuery(gtb_t *gtb, board_t *board, movelist_t *movelist){
    movelist->gencnt=0;
}

void GTBShow(gtb_t *gtb){
    OutputConsole("libgtb not present!\n");
}

void GTBResetStats(gtb_t *gtb){
    gtb->tbhits=0;
}

void GTBStatsShow(gtb_t *gtb){
    OutputConsole("TBHits: %uk\n",gtb->tbhits/1000);
}
#else 

#include <gtb-probe.h>


/* Most code below is adapted from "tbprobe.c" which comes with the 
 * gtb distribution. 
 */

#define GTB_AFTER_INIT 0
#define GTB_NOTREADY   1
#define GTB_READY      2



void GTBInit(gtb_t *gtb){
    // these are some default values
    gtb->verbosity = 1;
    gtb->scheme = tb_CP4;
    gtb->cache_size = 32*1024*1024;
    gtb->wdl_fraction = 96;
    gtb->path = tbpaths_init();
    gtb->state=GTB_AFTER_INIT;
    gtb->have_libgtb=true;
    gtb->tbhits=0;
}

void GTBSync(gtb_t *gtb){
    char *initinfo;	
    if(gtb->state==GTB_AFTER_INIT){
	initinfo = tb_init (gtb->verbosity, gtb->scheme, gtb->path);
	tbcache_init(gtb->cache_size, gtb->wdl_fraction);
	tbstats_reset();
	if (initinfo != NULL){
	    Log ("%s",initinfo);
	}
    }else if(gtb->state==GTB_NOTREADY){

	initinfo = tb_restart (gtb->verbosity, gtb->scheme, gtb->path);
	tbcache_restart(gtb->cache_size, gtb->wdl_fraction); 

	/* information to be output for the user, or to be saved in logs etc.*/
	if (initinfo != NULL){
		Log ("%s",initinfo);
	}
    }
    gtb->state=GTB_READY;
}

void GTBSetPath(gtb_t *gtb, char *path){
    gtb->path = tbpaths_done(gtb->path); 
    gtb->path = tbpaths_init();
    gtb->path =  tbpaths_add (gtb->path, path);
    if(gtb->state==GTB_READY){
	gtb->state=GTB_NOTREADY;
    }
    
}



void GTBSetCacheSize(gtb_t *gtb, size_t cache_size){
    gtb->cache_size=cache_size;
    if(gtb->state==GTB_READY){
	gtb->state=GTB_NOTREADY;
    }
}

void GTBSetWDLFraction(gtb_t *gtb, int wdl_fraction){
    gtb->wdl_fraction=wdl_fraction;
    if(gtb->state==GTB_READY){
	gtb->state=GTB_NOTREADY;
    }
}

void GTBSetScheme(gtb_t *gtb, int scheme){
    gtb->scheme=scheme;
    if(gtb->state==GTB_READY){
	gtb->state=GTB_NOTREADY;
    }

}

void GTBSetVerbosity(gtb_t *gtb, int verbosity){
    gtb->verbosity=verbosity;
    if(gtb->state==GTB_READY){
	gtb->state=GTB_NOTREADY;
    }
}


int GTBScore(gtb_t *gtb, board_t *board, int mode, int *flags){
    int stm;
    int epsquare;
    int castling;
    int i, sq;
    int pieces;
    BitBoard b;

    pieces=nbits(board->friends[white])+nbits(board->friends[black]);
    if(pieces>5){
	*flags=EG_NOTRECOGNIZED;
	return EG_SCORENOTRECOGNIZED;
    }
    
    // 5=4(pieces)+1(endmarker). 

    unsigned int  ws[5];
    unsigned int  bs[5];
    unsigned char wp[5];
    unsigned char bp[5];

    unsigned info=tb_UNKNOWN;
    unsigned pliestomate;
    int tb_available=0;

    if(gtb->state!=GTB_READY){
	GTBSync(gtb);
    }
    stm=(board->side==white)?tb_WHITE_TO_MOVE:tb_BLACK_TO_MOVE;
    epsquare=(board->ep==-1)?tb_NOSQUARE:board->ep;
    castling=tb_NOCASTLE;
    if(board->flag){
	if(board->flag & WKINGCASTLE){
	    castling|=tb_WOO;
	}
	if(board->flag & WQUEENCASTLE){
	    castling|=tb_WOOO;
	}
	if(board->flag & BKINGCASTLE){
	    castling|=tb_BOO;
	}
	if(board->flag & BQUEENCASTLE){
	    castling|=tb_BOOO;
	}
    }
    b=board->friends[white];
    i=0;
    while(b){
	sq=leadz(b);
	CLEARBIT(b,sq);
	ws[i]=sq;
	// By some miracle gnuchess and gtb use the same encoding for pieces. 
	wp[i]=board->cboard[sq];
	i++;
    }
    ws[i]=tb_NOSQUARE;
    wp[i]=tb_NOPIECE;

    b=board->friends[black];
    i=0;
    while(b){
	sq=leadz(b);
	CLEARBIT(b,sq);
	bs[i]=sq;
	bp[i]=board->cboard[sq];
	i++;
    }
    bs[i]=tb_NOSQUARE;
    bp[i]=tb_NOPIECE;

    switch(mode){
    case 0:   // hard,dtm
	tb_available=tb_probe_hard (stm, 
				    epsquare, 
				    castling, 
				    ws, 
				    bs, 
				    wp, 
				    bp, 
				    &info, 
				    &pliestomate);
	break;
    case GTB_MODE_WDL: //hard,wdl
	tb_available=tb_probe_WDL_hard (stm, 
					epsquare, 
					castling, 
					ws, 
					bs, 
					wp, 
					bp, 
					&info);
	pliestomate=MATE-GTB_WIN_SCORE;
	break;
    case GTB_MODE_SOFT: //soft,dtm
	tb_available=tb_probe_hard (stm, 
				    epsquare, 
				    castling, 
				    ws, 
				    bs, 
				    wp, 
				    bp, 
				    &info, 
				    &pliestomate);
	break;

    case GTB_MODE_SOFT|GTB_MODE_WDL: //soft,wdl
	tb_available=tb_probe_WDL_soft (stm, 
					epsquare, 
					castling, 
					ws, 
					bs, 
					wp, 
					bp, 
					&info);
	pliestomate=MATE-GTB_WIN_SCORE;
	break;

	
    }
    if(!tb_available){
	*flags=EG_NOTRECOGNIZED;
	return EG_SCORENOTRECOGNIZED;
    }else{
	gtb->tbhits++;
	*flags=EG_EXACTSCORE;
	switch(info){
	case tb_DRAW:
	    return DRAWSCORE;
	case tb_WMATE:
	    return board->side==white?MATE+1-pliestomate:-MATE-1+pliestomate;
	case tb_BMATE:
	    return board->side==black?MATE+1-pliestomate:-MATE-1+pliestomate;
	}
    }
    ASSERT(0);
    return 0;
}

void GTBResetStats(gtb_t *gtb){
    gtb->tbhits=0;
}

void GTBQuery(gtb_t *gtb, board_t *board, movelist_t *movelist){
    movelist_t legal[1];
    undo_t undo[1];
    leaf *p;
    int gtb_probe;
    int flags;
    movelist->gencnt=0;
    if(gtb->state!=GTB_READY){
	GTBSync(gtb);
    }
    GenMoves(board,legal);
    for(p = legal->moves; p < legal->moves+legal->gencnt; p++){
	MoveMakeS(board,&(p->move),undo);
	gtb_probe=GTBScore(gtb,board,0/*hard,dtm*/,&flags);
	if(flags==EG_EXACTSCORE){
	    p->score=DRAWSCORE;
	    if(gtb_probe!=DRAWSCORE){
		p->score=gtb_probe>0?-gtb_probe+1:-gtb_probe-1;
	    }
	    movelist->moves[movelist->gencnt++]=*p;
	}
	MoveUnMakeS(board,undo);
    }
    
}

void GTBShow(gtb_t *gtb){
    int av;
    if(gtb->state!=GTB_READY){
	GTBSync(gtb);
    }

    av = tb_availability();
    
    if (0 != (av& 2)){
	OutputConsole ("3-pc GTBs complete\n");  
    }else if (0 != (av& 1)) {
	OutputConsole("Some 3-pc GTBs available\n"); 
    }else{
	OutputConsole("No 3-pc GTBs available\n");
    }

    if(0 != (av& 8)){
	OutputConsole ("4-pc GTBs complete\n");  
    }else if (0 != (av& 4)){ 
	OutputConsole ("Some 4-pc GTBs available\n"); 
    }else{ 
	OutputConsole ("No 4-pc GTBs available\n");
    }

    if (0 != (av&32)){
	OutputConsole ("5-pc GTBs complete\n");
    }else if (0 != (av&16)){
	OutputConsole ("Some 5-pc GTBs available\n");}
    else {
	OutputConsole ("No 5-pc GTBs available\n");
    }
}

void GTBStatsShow(gtb_t *gtb){
    OutputConsole("TBHits: %uk\n",gtb->tbhits/1000);
}

#endif // HAVE_LIBGTB
