/* GNU Chess 5.0 - hung.c - hung piece evaluation code
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   GNU Chess is based on the two research programs 
   Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.

   GNU Chess is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GNU Chess is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info: 
     bug-gnu-chess@gnu.org
     cracraft@ai.mit.edu, cracraft@stanfordalumni.org, cracraft@earthlink.net
*/
/*
 *  Hung piece evaluation.
 */

#include <stdio.h>
#include "piece.h"
#include "board.h"
#include "attack.h"
#include "hung.h"
#include "log.h"

BitBoard HungGen(board_t *board, attack_t *attack, int side)
/********************************************************************
 *
 * This function returns a bitboard with hung pieces for side.
 *
 ********************************************************************/
{
   BitBoard n, b, r, q, hung;
   int xside;
   //   int hunged[2];

   ASSERT(AttackValidate(board,attack));

   xside = 1 ^ side;
   // hunged[side] = 0;

   /* Knight */
   n = (attack->b[xside][pawn] & board->b[side][knight]);
   n |= (attack->b[xside][0] & board->b[side][knight] & ~(attack->b[side][0]));

   /* Bishop */
   b = (attack->b[xside][pawn] & board->b[side][bishop]);
   b |= (attack->b[xside][0] & board->b[side][bishop] & ~(attack->b[side][0]));

   /* Rook */
   r = attack->b[xside][pawn] | attack->b[xside][knight] | attack->b[xside][bishop];
   r = (r & board->b[side][rook]);
   r |= (attack->b[xside][0] & board->b[side][rook] & ~(attack->b[side][0]));

   /* Queen */
   q = attack->b[xside][pawn] | attack->b[xside][knight] | attack->b[xside][bishop] |
       attack->b[xside][rook];
   q = (q & board->b[side][queen]);
   q |= (attack->b[xside][0] & board->b[side][queen] & ~(attack->b[side][0]));

   hung = n | b | r | q ;
   return hung;
}


int HungCount(board_t *board, attack_t *attack, int side)
/****************************************************************************
 *
 *  Calculate the number of hung pieces for side.
 *
 ****************************************************************************/
{
   BitBoard c, n, b, r, q;
   int xside;
   int hunged[2];

   ASSERT(AttackValidate(board,attack));

   xside = 1 ^ side;
   hunged[side] = 0;

   /* Knight */
   n = (attack->b[xside][pawn] & board->b[side][knight]);
   n |= (attack->b[xside][0] & board->b[side][knight] & ~(attack->b[side][0]));

   /* Bishop */
   b = (attack->b[xside][pawn] & board->b[side][bishop]);
   b |= (attack->b[xside][0] & board->b[side][bishop] & ~(attack->b[side][0]));

   /* Rook */
   r = attack->b[xside][pawn] | attack->b[xside][knight] | attack->b[xside][bishop];
   r = (r & board->b[side][rook]);
   r |= (attack->b[xside][0] & board->b[side][rook] & ~(attack->b[side][0]));

   /* Queen */
   q = attack->b[xside][pawn] | attack->b[xside][knight] | attack->b[xside][bishop] |
       attack->b[xside][rook];
   q = (q & board->b[side][queen]);
   q |= (attack->b[xside][0] & board->b[side][queen] & ~(attack->b[side][0]));

   c = n | b | r | q ;

   if (c)
      hunged[side] += nbits (c);

   /* King */
   if (attack->b[xside][0] & board->b[side][king])
      hunged[side] ++;

   return (hunged[side]);
}
