#include <string.h>

#include "util.h"
#include "eval.h"
#include "material.h"
#include "piece.h"
#include "log.h"

int MaterialDelta[2][8];
material_slot_t  Material[MATERIAL_SIZE];

static void MaterialDeltaInit(void){
    MaterialDelta[white][queen]=                     1;
    MaterialDelta[white][rook]=                      2;
    MaterialDelta[white][knight]=                  3*2;
    MaterialDelta[white][bishop]=                3*3*2;
    MaterialDelta[white][pawn]=                3*3*3*2;

    MaterialDelta[black][queen]=             9*3*3*3*2;
    MaterialDelta[black][rook]=            2*9*3*3*3*2;
    MaterialDelta[black][knight]=        3*2*9*3*3*3*2;
    MaterialDelta[black][bishop]=      3*3*2*9*3*3*3*2;
    MaterialDelta[black][pawn]=      3*3*3*2*9*3*3*3*2;
}



void MaterialFromIndex(int material[2][8],int index){
    int saveindex;
    saveindex=index;

    material[white][queen]=   index%2;
    index/=2;
    material[white][rook] =   index%3;
    index/=3;
    material[white][knight] = index%3;
    index/=3;
    material[white][bishop] = index%3;
    index/=3;
    material[white][pawn] =   index%9;
    index/=9;

    material[black][queen]=   index%2;
    index/=2;
    material[black][rook] =   index%3;
    index/=3;
    material[black][knight] = index%3;
    index/=3;
    material[black][bishop] = index%3;
    index/=3;
    material[black][pawn] =   index%9;

    ASSERT((index/9)==0);;
    ASSERT(saveindex==MaterialToIndex(material));
}

int MaterialToIndex(int material[2][8]){
    int color;
    enum Piece piece;
    int sum=0;
    for(color=white;color<=black;color++){
	for(piece=pawn;piece<=queen;piece++){
	    sum+=material[color][piece]*MaterialDelta[color][piece];
	}
    }
    return sum;
}


void MaterialInit(void){
    int index;
    int color, xcolor;
    int material[2][8];
    int m[2];
    int mm[2];
    int piece;
    int piece_weight;

    MaterialDeltaInit();

    memset(Material,0,sizeof(Material));
    
    for(index=0;index<MATERIAL_SIZE;index++){
	MaterialFromIndex(material,index);
	for(color=white;color<=black;color++){
	    m[color]=
		material[color][pawn]   *evals.ValueP+
		material[color][knight] *evals.ValueN+
		material[color][bishop] *evals.ValueB+
		material[color][rook]   *evals.ValueR+
		material[color][queen]  *evals.ValueQ;
	    mm[color]=
		material[color][pawn]   * 0 +
		material[color][knight] * 1 +
		material[color][bishop] * 1 +
		material[color][rook]   * 2 +
		material[color][queen]  * 4;
	}
	// temporary
	Material[index].phase= (MAX(0,(8 - (m[white]+m[black]) / 1150)));
	
	ASSERT(((48-(mm[white]+mm[black]))*255+24)/48<=255);
	ASSERT(((48-(mm[white]+mm[black]))*255+24)/48>=0);

	Material[index].phase256= ((24-(mm[white]+mm[black]))*255+12)/24;

	ASSERT(Material[index].phase256==((24-(mm[white]+mm[black]))*255+12)/24);

	// feeble attempt to encourage trading down pieces and not pawns when we're ahead

	piece_weight=0;
	for(piece=knight;piece<=queen;piece++){
	    piece_weight-=(material[white][piece]+material[black][piece])*evals.tradepiece;
	}
	piece_weight+=(material[white][pawn]+material[black][pawn])*evals.tradepawns;

	if(m[white]-m[black]>=200){
	    Material[index].score+=piece_weight;
	}else if(m[white]-m[black]<=-200){
	    Material[index].score-=piece_weight;
	}

	/* only pawns */
	if(m[white]==material[white][pawn]*evals.ValueP 
	   && 
	   m[black]==material[black][pawn]*evals.ValueP){
	    Material[index].flags|=MATERIAL_PAWN_ENDGAME;
	}

	/* no mating material */
	if(material[white][pawn]==0  && (m[white] < evals.ValueR || 
					 (m[white] == 2*evals.ValueN 
					  && material[white][knight]==2))){
	    Material[index].flags|=MATERIAL_WHITE_NO_MATING;
	}                                                                

	if(material[black][pawn]==0  && (m[black] < evals.ValueR || 
					 (m[black] == 2*evals.ValueN 
					  && material[black][knight]==2))){
	    Material[index].flags|=MATERIAL_BLACK_NO_MATING;
	}                                                                

	if(material[white][pawn]==0 && 
	   material[black][pawn]==1 && 
	   material[white][bishop]+material[white][knight]>=1 &&
	   m[black]-m[white] <= evals.ValueP){
	    Material[index].flags|=MATERIAL_WHITE_CAN_SAC;
		   
	}

	if(material[black][pawn]==0 && 
	   material[white][pawn]==1 && 
	   material[black][bishop]+material[black][knight]>=1 &&
	   m[white]-m[black] <= evals.ValueP){
	    Material[index].flags|=MATERIAL_BLACK_CAN_SAC;
		   
	}
	   



	for(color=white;color<=black;color++){
	    xcolor=1^color;

	    // should be else if's but this is more readable

	    if(m[color]==evals.ValueP && m[xcolor]==0){
		Material[index].recog=MATERIAL_KPK;
	    }

	    // P_ means: zero or more pawns

	    if(m[color]>evals.ValueP && m[color]==material[color][pawn]*evals.ValueP && m[xcolor]==0){
		Material[index].recog=MATERIAL_KPPP_K;
	    }

	    if(m[color]>evals.ValueP && m[color]==material[color][pawn]*evals.ValueP && m[xcolor]>=evals.ValueP && m[xcolor]==material[xcolor][pawn]*evals.ValueP){
		Material[index].recog=MATERIAL_KPPP_KPP_;
	    }

	    // FIXME: KBPK means really KBPKP_
	    if(m[color]==evals.ValueB+material[color][pawn]*evals.ValueP && material[color][bishop]==1 && m[xcolor]==material[xcolor][pawn]*evals.ValueP){
		Material[index].recog=MATERIAL_KBPK;
	    }
	    if(m[color]==evals.ValueQ && material[color][queen]==1 && m[xcolor]==evals.ValueP){
		Material[index].recog=MATERIAL_KQKP;
	    }
	    if(m[color]==evals.ValueN && material[color][knight]==1 && m[xcolor]==0){
		Material[index].recog=MATERIAL_KNK;
	    }
	    if(m[color]==evals.ValueB && material[color][bishop]==1 && m[xcolor]==0){
		Material[index].recog=MATERIAL_KBK;
	    }
	    if(m[color]==evals.ValueB+evals.ValueN && material[color][bishop]==1 && material[color][knight]==1 && m[xcolor]==0){
		Material[index].recog=MATERIAL_KBNK;
	    }
	    if(m[color]==evals.ValueN+evals.ValueN && material[color][knight]==1 && m[xcolor]==0){
		Material[index].recog=MATERIAL_KNNK;
	    }
	    if(m[color]==evals.ValueB && material[color][bishop]==1 && m[xcolor]==evals.ValueB && material[xcolor][bishop]==1){
		Material[index].recog=MATERIAL_KBKB;
	    }
	    if(m[color]==evals.ValueB && material[color][bishop]==1 && m[xcolor]==evals.ValueN && material[xcolor][knight]==1){
		Material[index].recog=MATERIAL_KBKN;
	    }
	    if(m[color]==evals.ValueN && material[color][knight]==1 && m[xcolor]==evals.ValueN && material[xcolor][knight]==1){
		Material[index].recog=MATERIAL_KNKN;
	    }
	    if(m[color]==evals.ValueR+evals.ValueP && material[color][rook]==1 && m[xcolor]==evals.ValueR && material[xcolor][rook]==1){
		Material[index].recog=MATERIAL_KRPKR;
	    }
	    if(m[color]==evals.ValueR && material[color][rook]==1 && m[xcolor]==evals.ValueP && material[xcolor][pawn]==1){
		Material[index].recog=MATERIAL_KRKP;
	    }
	    if(Material[index].recog!=0 && color==black){
		Material[index].flags|=MATERIAL_WINNER_BLACK;
	    }
	}	
	
    }
}
