#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "board.h"
#include "movelist.h"
#include "move.h"
#include "piece.h"
#include "attack.h"
#include "log.h"


// functions


void MoveListInit (movelist_t *movelist){
    movelist->gencnt=0;
}

int MoveListContains(movelist_t *movelist, int move){
    int i;
    for(i=0;i<movelist->gencnt;i++){
	if((movelist->moves[i].move & MOVEMASK) == (move & MOVEMASK)){
	    return true;
	}
    }
    return false;
}

leaf * MoveListFind(movelist_t *movelist, int move){
    int i;
    for(i=0;i<movelist->gencnt;i++){
	if((movelist->moves[i].move & MOVEMASK) == (move & MOVEMASK)){
	    return movelist->moves+i;
	}
    }
    return NULL;
}

void MoveListFromString(board_t *board, movelist_t *movelist, char *s){
/************************************************
 *
 * This parses list of move score pairs. 
 * If score is not present it is set to -INFIN,
 *   
 ************************************************/
    char *ss, *s1;
    int score;
    int move;
    char c;

    MoveListInit(movelist);
    
    ss=strdup(s);
    s1=strtok(ss," ,=\r\n");
    while(s1!=NULL){
	if(isdigit(*s1)){
	    score=0;
	    while((c=*(s1++))){
		score=10*score+c-'0';
	    }
	    if(movelist->gencnt>0){
		movelist->moves[movelist->gencnt-1].score=score;
	    }
	}else{
	    MoveFromString(board,&move,s1);
	    if(move!=NOMOVE){
		movelist->moves[movelist->gencnt].move=move;
		movelist->moves[movelist->gencnt++].score=NOSCORE;
	    }
	    
	}
	s1=strtok(NULL," ,=\r\n");
    }
    free(ss);
}


void MoveListFilter (board_t *board,  movelist_t *movelist)
/**************************************************************************
 *
 *  All the illegal moves is the list are removed
 *
 **************************************************************************/
{
   leaf *p;
   int side, xside;
   int check, sq;
   undo_t undo[1];

   side = board->side;
   xside = 1^side;
   sq = board->king[side];
   for (p = movelist->moves; p < movelist->moves+movelist->gencnt; p++) {
     MoveMakeS(board, &p->move, undo);
     if (board->cboard[TOSQ(p->move)] != king){
       check = AttackToSq (board,sq, xside); 
     }else {
       check = AttackToSq (board,TOSQ(p->move), xside); 
     }
     MoveUnMakeS (board, undo);
     if (check)	 { /* its an illegal move */
       --(movelist->gencnt);
       *p=*(movelist->moves+movelist->gencnt);
       --p; 
     } 
   }
   ASSERT(movelist->gencnt>=0);
}

void MoveListAdd(board_t *board, movelist_t *movelist, leaf *p){
    ASSERT(MoveIsPseudoLegal(board,p->move));
    movelist->moves[movelist->gencnt]=*p;
    movelist->gencnt++;
}


void MoveListToString(board_t *board, movelist_t *movelist,  char *s){
    leaf *node;
    char SANmv[SANSZ];
    char tmp[MAXSTR];
    
    s[0]='\0';

    for (node = movelist->moves; node < movelist->moves+movelist->gencnt; node++) {
	if(node!=movelist->moves){
	    sprintf(tmp,"  ");
	    strcat(s,tmp);
	}
	if(board!=NULL){
	    ASSERT(MoveIsPseudoLegal(board,node->move));
	    MoveToSAN (board,node->move,SANmv);
	}else{
	    MoveToLAN (board,node->move,SANmv);
	}
	if(node->score!=NOSCORE){
	    sprintf(tmp,"%s=%d",SANmv,node->score);
	}else{
	    sprintf(tmp,"%s",SANmv);
	}
	strcat(s,tmp);
    }
}

void MoveListShow (board_t *board, movelist_t *movelist)
/**************************************************************************
 *
 *  Print out the move list.  
 *
 **************************************************************************/
{
   char s[MAXSTR];

   MoveListToString(board,movelist,s);
   Output("%s\n",s);
   return;
} 

void MoveListCopy(movelist_t *dst_movelist, movelist_t *src_movelist){
    memcpy(dst_movelist, src_movelist, sizeof(movelist_t));
}

leaf * MoveListPickUniform(movelist_t *movelist){
    int r;
    int k;
    r=rand();
    k=r%movelist->gencnt;
    return movelist->moves+k;
}

leaf * MoveListPickRandom(movelist_t *movelist){
  int i,k;
  int sum;
  int r;
  sum=0;
  for(i=0;i<movelist->gencnt;i++){
    sum+=movelist->moves[i].score;
  }
  r=rand();
  if(sum==0){
    return NULL;
  }
  k=r%sum;
  sum=0;
  for(i=0;i<movelist->gencnt;i++){
    sum+=movelist->moves[i].score;
    if(sum>k){
      break;
    }
  }
  return &(movelist->moves[i]);
}

leaf * MoveListPickBest(movelist_t *movelist){
  int i;
  int best;
  int ibest;
  if(movelist->gencnt==0){
      return NULL;
  }
  best=-INFIN-1;
  ibest=0;
  for(i=0;i<movelist->gencnt;i++){
    if(movelist->moves[i].score>best){
      best=movelist->moves[i].score;
      ibest=i;
    }
  }
  return movelist->moves+ibest;
}

void MoveListPickSel(movelist_t *movelist, leaf *head)
/***************************************************************************
 *
 *  This pick routine searches the movelist and swap the high score entry
 *  with the one currently at the head.
 *
 ***************************************************************************/
{
   int best;
   leaf *p, *pbest, tmp;

   best = head->score;
   pbest = head;
   for (p = head+1; p < movelist->moves+movelist->gencnt; p++) {
      if (p->score > best) {
         pbest = p;
	 best = p->score;
      }
   }

   /*  Swap pbest with the head  */
   tmp = *head;
   *head = *pbest;
   *pbest = tmp; 
}

