#include "movelist.h"
#include "move.h"
#include "perft.h"
#include "input.h"
#include "genmove.h"
#include "ttable.h"

/* The idea for doing perft by sampling
   exactly one move at each ply is due to
   Peter Osterlund.
*/

double PerftMCUniformOnce(board_t *board, 
			  int full_depth, 
			  int depth, 
			  uint64_t *nodes){
    movelist_t movelist[1];
    undo_t undo[1];
    leaf *p;
    double  total;
    if(InputLook()){
	return 0;
    }
    (*nodes)++;
    GenMoves(board,movelist);

    if(depth<=0){
	return 1;
    }else if(depth==1 || movelist->gencnt==0){
	return movelist->gencnt;
    }else if(full_depth>0){
	total=0;
	for(p=movelist->moves;p<movelist->moves+movelist->gencnt;p++){
	    MoveMakeS(board,&p->move,undo);
	    total+=PerftMCUniformOnce(board,full_depth-1,depth-1,nodes);
	    MoveUnMakeS(board,undo);
	}
	return total;
    }else{
	p=MoveListPickUniform(movelist);
	MoveMakeS(board,&p->move,undo);
	total=movelist->gencnt*PerftMCUniformOnce(board,0,depth-1,nodes);
	MoveUnMakeS(board,undo);
	return total;
    }
}



uint64_t Perft(board_t *board, int d, uint64_t *nodes, int hash){
    movelist_t movelist[1];
    undo_t undo[1];
    leaf *p;
    uint64_t total;
    (*nodes)++;
    if(hash && TTablePerftGet(TTable,board,d,&total)){
	return total;
    }
    
    if(board->in_check){
	GenPseudoCheckEscapes(board,movelist);
    }else{
	GenPseudoMoves(board,movelist);
    }

    if(d<=0){
        total=1;
    }else if(d==1){
	MoveListFilter(board,movelist);
	total=movelist->gencnt;
    }else{
	total=0;
	for(p=movelist->moves;p<movelist->moves+movelist->gencnt;p++){
	    MoveMakeS(board,&p->move,undo);
	    if(board->xin_check){
		MoveUnMakeS(board,undo);
		continue;
	    }
	    total+=Perft(board,d-1,nodes,hash);
	    MoveUnMakeS(board,undo);
	}
    }
    if(hash){
	TTablePerftPut(TTable,board,d,total);
    }
    return total;
} 

void PerftDiv(board_t *board, movelist_t *movelist, int d, int hash){
    leaf *p;
    undo_t undo[1];
    uint64_t nodes;
    GenMoves(board,movelist);
    for(p=movelist->moves;p<movelist->moves+movelist->gencnt;p++){
	MoveMakeS(board,&p->move,undo);
	p->score=Perft(board,d-1,&nodes,hash);
	MoveUnMakeS(board,undo);
    }

}
