
// pst.cpp

// This file is adapted from pst.cpp in the Fruit source code.

// includes

#include <stdio.h>

#include "square.h"
#include "piece.h"
#include "util.h"
#include "option.h"

// macros

#define P(piece,color,square_64,stage) (Pst[(piece)][(color)][(square_64)][(stage)])

// constants and variables

typedef struct {
    int PieceSquareWeight;
    int PieceActivityWeight;
    int KingSafetyWeight;
    int PawnStructureWeight;

    int PawnFileOpening;
    int KnightCentreOpening;
    int KnightCentreEndgame;
    int KnightRankOpening;
    int KnightBackRankOpening;
    int KnightTrapped;
    int BishopCentreOpening;
    int BishopCentreEndgame;
    int BishopBackRankOpening;
    int BishopDiagonalOpening;
    int RookFileOpening;
    int QueenCentreOpening;
    int QueenCentreEndgame;
    int QueenBackRankOpening;
    int KingCentreEndgame;
    int KingFileOpening;
    int KingRankOpening;

    int T1;
    int T2;
    int T3;
    int T4;
    int T5;

} pst_t;

pst_t pst;

// "constants"

static const int PawnFile[8] = {
   -3, -1, +0, +1, +1, +0, -1, -3,
};

static const int KnightLine[8] = {
   -4, -2, +0, +1, +1, +0, -2, -4,
};

static const int KnightRank[8] = {
   -2, -1, +0, +1, +2, +3, +2, +1,
};

static const int BishopLine[8] = {
   -3, -1, +0, +1, +1, +0, -1, -3,
};

static const int RookFile[8] = {
   -2, -1, +0, +1, +1, +0, -1, -2,
};

static const int QueenLine[8] = {
   -3, -1, +0, +1, +1, +0, -1, -3,
};

static const int KingLine[8] = {
   -3, -1, +0, +1, +1, +0, -1, -3,
};

static const int KingFile[8] = {
   +3, +4, +2, +0, +0, +2, +4, +3,
};

static const int KingRank[8] = {
   +1, +0, -2, -3, -4, -5, -6, -7,
};

// variables

int Pst[8][2][64][2];

// functions

// PSTShow()

void PSTShow(int piece, int color, int stage){
    int f,r;
    for(r=7;r>=0;r--){
	for(f=0;f<=7;f++){
	    printf("%5d",P(piece,color,SQUARE(f,r),stage));
	}
	printf("\n");
    }
}

// PSTInit()

void PSTInit(option_list_t *options) {

   int i;
   int piece, sq, stage;
   int color;

   pst.T1=OptionGetInt(options,"T1");
   pst.T2=OptionGetInt(options,"T2");
   pst.T3=OptionGetInt(options,"T3");
   pst.T4=OptionGetInt(options,"T4");
   pst.T5=OptionGetInt(options,"T5");


   pst.PieceSquareWeight = 256; // 100%
   pst.PieceActivityWeight = 256; // 100%
   pst.KingSafetyWeight = 256; // 100%
   pst.PawnStructureWeight = 256; // 100%

   pst.PawnFileOpening = 5;
   pst.KnightCentreOpening = 5;
   pst.KnightCentreEndgame = 5;
   pst.KnightRankOpening = 5;
   pst.KnightBackRankOpening = 0;
   pst.KnightTrapped = 100;
   pst.BishopCentreOpening = 2;
   pst.BishopCentreEndgame = 3;
   pst.BishopBackRankOpening = 10;
   pst.BishopDiagonalOpening = 4;
   pst.RookFileOpening = 3;
   pst.QueenCentreOpening = 0; // was 0
   pst.QueenCentreEndgame = 4;
   pst.QueenBackRankOpening = 5;
   //   pst.KingCentreEndgame = 12;
   //   pst.KingCentreEndgame = pst.T1;  // multiples of 256
   pst.KingCentreEndgame = 4362;  // multiples of 256
   pst.KingFileOpening = 10;
   pst.KingRankOpening = 10;
   

   // init

   for (piece = pawn; piece <=king; piece++) {
       for(color=white; color<=black; color++){
	   for (sq = 0; sq < 64; sq++) {
	       for (stage = 0; stage<=1; stage++) {
		   P(piece,color,sq,stage) = 0;
	       }
	   }
       }
   }



   // pawns

   piece = pawn;
   color = white;

   // file

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) += PawnFile[FILE_(sq)] * pst.PawnFileOpening;
   }

   // centre control

   P(piece,color,D3,0) += 10;
   P(piece,color,E3,0) += 10;

   P(piece,color,D4,0) += 20;
   P(piece,color,E4,0) += 20;

   P(piece,color,D5,0) += 10;
   P(piece,color,E5,0) += 10;

   // weight

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) = (P(piece,color,sq,0) * pst.PawnStructureWeight) / 256;
       P(piece,color,sq,0) = (P(piece,color,sq,0) * pst.PawnStructureWeight) / 256;
   }


   // knights

   piece = knight;
   color = white;

   // centre

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) += KnightLine[FILE_(sq)] * pst.KnightCentreOpening;
       P(piece,color,sq,0) += KnightLine[RANK(sq)] * pst.KnightCentreOpening;
       P(piece,color,sq,1) += KnightLine[FILE_(sq)] * pst.KnightCentreEndgame;
       P(piece,color,sq,1) += KnightLine[RANK(sq)] * pst.KnightCentreEndgame;
   }

   // rank

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) += KnightRank[RANK(sq)] * pst.KnightRankOpening;
   }

   // back rank

   for (sq = A1; sq <= H1; sq++) { // HACK: only first rank
       P(piece,color,sq,0) -= pst.KnightBackRankOpening;
   }

   // "trapped"

   P(piece,color,A8,0) -= pst.KnightTrapped;
   P(piece,color,H8,0) -= pst.KnightTrapped;

   // weight

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) = (P(piece,color,sq,0) * pst.PieceSquareWeight) / 256;
       P(piece,color,sq,1) = (P(piece,color,sq,1) * pst.PieceSquareWeight) / 256;
   }

   // bishops

   piece = bishop;
   color = white;

   // centre

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) += BishopLine[FILE_(sq)] * pst.BishopCentreOpening;
       P(piece,color,sq,0) += BishopLine[RANK(sq)] * pst.BishopCentreOpening;
       P(piece,color,sq,1) += BishopLine[FILE_(sq)] * pst.BishopCentreEndgame;
       P(piece,color,sq,1) += BishopLine[RANK(sq)] * pst.BishopCentreEndgame;
   }

   // back rank

   for (sq = A1; sq <= H1; sq++) { // HACK: only first rank
       P(piece,color,sq,0) -= pst.BishopBackRankOpening;
   }

   // main diagonals

   for (i = 0; i < 8; i++) {
       sq = SQUARE(i,i);
       P(piece,color,sq,0) += pst.BishopDiagonalOpening;
       sq = SQUARE(i,7-i);
       P(piece,color,sq,0) += pst.BishopDiagonalOpening;
   }

   // weight

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) = (P(piece,color,sq,0) * pst.PieceSquareWeight) / 256;
       P(piece,color,sq,1) = (P(piece,color,sq,1) * pst.PieceSquareWeight) / 256;
   }

   // rooks

   piece = rook;
   color = white;

   // file

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) += RookFile[FILE_(sq)] * pst.RookFileOpening;
   }

   // weight

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) = (P(piece,color,sq,0) * pst.PieceSquareWeight) / 256;
       P(piece,color,sq,1) = (P(piece,color,sq,1) * pst.PieceSquareWeight) / 256;
   }

   // queens

   piece = queen;
   color = white;

   // centre

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) += QueenLine[FILE_(sq)] * pst.QueenCentreOpening;
       P(piece,color,sq,0) += QueenLine[RANK(sq)] * pst.QueenCentreOpening;
       P(piece,color,sq,1) += QueenLine[FILE_(sq)] * pst.QueenCentreEndgame;
       P(piece,color,sq,1) += QueenLine[RANK(sq)] * pst.QueenCentreEndgame;
   }

   // back rank

   for (sq = A1; sq <= H1; sq++) { // HACK: only first rank
       P(piece,color,sq,0) -= pst.QueenBackRankOpening;
   }

   // weight

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) = (P(piece,color,sq,0) * pst.PieceSquareWeight) / 256;
       P(piece,color,sq,1) = (P(piece,color,sq,1) * pst.PieceSquareWeight) / 256;
   }

   // kings

   piece = king;
   color = white;

   // centre

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,1) += (KingLine[FILE_(sq)] * pst.KingCentreEndgame)/256;
       P(piece,color,sq,1) += (KingLine[RANK(sq)] * pst.KingCentreEndgame)/256;
   }

   // file

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) += KingFile[FILE_(sq)] * pst.KingFileOpening;
   }

   // rank

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) += KingRank[RANK(sq)] * pst.KingRankOpening;
   }

   // weight

   for (sq = 0; sq < 64; sq++) {
       P(piece,color,sq,0) = (P(piece,color,sq,0) * pst.KingSafetyWeight)    / 256;
       P(piece,color,sq,1) = (P(piece,color,sq,1) * pst.PieceSquareWeight) / 256;
   }

   // symmetry copy for black

   for (piece = pawn; piece <=king; piece ++) { // HACK
       for (sq = 0; sq < 64; sq++) {
	   for (stage = 0; stage < 2; stage++) {
	       P(piece,black,sq,stage) = -P(piece,
					    white,
					    SQUARE(FILE_(sq),7-RANK(sq)),
					    stage); // HACK
	   }
       }
   }
}

