#include "stdlib.h"
#include "searchparams.h"
#include "util.h"
#include "protocol.h"
#include "option.h"
#include "uci.h"


void SearchParamsShow(searchparams_t *searchparams){
    OutputConsole("protocol=%d\n",searchparams->protocol);
    OutputConsole("TCinc=%f\n",searchparams->TCinc);
    OutputConsole("SearchDepth=%d\n",searchparams->SearchDepth);
    OutputConsole("ST=%f\n",searchparams->ST);
    OutputConsole("movestogo=%d\n",searchparams->movestogo);
    OutputConsole("NPS=%d\n",searchparams->NPS);
    OutputConsole("infinite=%d\n",searchparams->infinite);
    OutputConsole("ponder=%d\n",searchparams->ponder);
    OutputConsole("post=%d\n",searchparams->post);
    OutputConsole("SearchTime=%f\n",searchparams->SearchTime);
    OutputConsole("maxtime=%f\n",searchparams->maxtime);
    OutputConsole("timelimit=%f\n",searchparams->timelimit);
    OutputConsole("movelist=\n");
    MoveListShow(NULL,searchparams->movelist);
    OutputConsole("listen=%f\n",searchparams->movelist);
}

void SearchParamsCreate(searchparams_t *searchparams, 
			protocol_t *protocol){
    board_t board[1];
    BoardFromGame(board,protocol->game,BOARD_LAST);
    searchparams->protocol=PROTOCOL_XBOARD;
    // note: floats can represent literal zeroes
    searchparams->TCinc=(float) protocol->TCinc; 
    searchparams->SearchDepth=protocol->SearchDepth;
    searchparams->timelimit=protocol->TimeLimit[board->side];
    searchparams->NPS=protocol->NPS;
    // note: floats can represent literal zeroes
    searchparams->ST=(float) protocol->ST; 
    searchparams->SearchTime=0.0;
    searchparams->maxtime=0.0;
    searchparams->infinite=false;
    searchparams->ponder=false;
    searchparams->post=(protocol->state_flags & POST)?true:false;
    searchparams->nodes=0;
    if(protocol->TCMove){
	searchparams->movestogo=(protocol->NextPeriod-board->GameCnt+1)/2;
    }else{
	searchparams->movestogo=0;
    }
    searchparams->pondering=false; // no ponder time management in xboard move (yet).
    MoveListInit(searchparams->movelist);
    searchparams->listen=true;
}

void SearchParamsCreateUci(searchparams_t *searchparams,
			   uci_t *uci,
			   option_list_t *option){
    searchparams->protocol=PROTOCOL_UCI;
    searchparams->movestogo=uci->movestogo;
    searchparams->timelimit=((float)(uci->board->side==white?uci->wtime:uci->btime))/1000;
    // note: floats can represent literal zeroes
    searchparams->TCinc=((float)(uci->board->side==white?uci->winc:uci->binc))/1000;
    // note: floats can represent literal zeroes
    searchparams->ST=((float) uci->movetime)/1000;
    searchparams->SearchDepth=uci->depth;
    searchparams->infinite=uci->infinite;
    searchparams->ponder=uci->ponder;
    searchparams->nodes=uci->nodes;
    if(uci->ponder){
	searchparams->infinite=true;
    }
    searchparams->pondering=OptionGetBool(option,"Ponder"); 
    MoveListCopy(searchparams->movelist,uci->movelist);
    searchparams->listen=true;

}
