/* GNU Chess 5.0 - solve.c - position solving code
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   GNU Chess is based on the two research programs 
   Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.

   GNU Chess is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GNU Chess is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info: 
     bug-gnu-chess@gnu.org
     cracraft@ai.mit.edu, cracraft@stanfordalumni.org, cracraft@earthlink.net
*/
/* 
 *
 */

#include <stdio.h>
#include <string.h>
#include "util.h"
#include "epd.h"
#include "searchparams.h"
#include "searchstats.h"
#include "iterate.h"
#include "move.h"
#include "ttable.h"
#include "ptable.h"
#include "log.h"

void SolveSTS (char *file, searchparams_t *searchparams)
/*****************************************************************************
 *
 *
 *
 *****************************************************************************/
{

   searchstats_t searchstats[1];
   epd_iter_t epd_iter[1];
   epd_t epd[1];

   int total, correct, found;
   long TotalNodes;
   char SANmv[SANSZ];
   movelist_t sts[1];
   int score;
   int total_score;
   int max_score;
   int this_max_score;
   int i;


   total = correct = 0;
   total_score=0;
   max_score=0;
   TotalNodes = 0;
   EPDIterInit(epd_iter);
   EPDIterStart(epd_iter,file);
   if(searchparams->ST==0){
       searchparams->ST=10;
   }
   while (!(Protocol->state_flags & QUIT) && EPDIterNext(epd_iter,epd)) {
     TTableZero(TTable);  
     PTableZero(PTable);  
     EPDShow(epd);
     MoveListFromString(epd->board,sts,epd->c0);
     total++;

     SearchStatsInit(searchstats);
     Iterate (epd->board,searchparams,searchstats);
     TotalNodes += searchstats->NodeCnt + searchstats->QuiesCnt;
     MoveToSAN(epd->board,searchstats->RootPV,SANmv);
     found = false;
     if(MoveListContains(epd->bm,searchstats->RootPV)){
	 correct+=1;
	 found=true;
     }
     score=0;
     this_max_score=0;
     for(i=0;i<sts->gencnt;i++){
	 score=sts->moves[i].score;
	 if(score>this_max_score){
	     this_max_score=score;
	 }
	 if((searchstats->RootPV & MOVEMASK)==(sts->moves[i].move & MOVEMASK)){
	     total_score+=score;
	 }

     }
     max_score+=this_max_score;
     Output ("\nmove=%s ", SANmv);
     Output (found ? "(Correct)\n" : "(Incorrect)\n");
     Output ("Correct=%d Total=%d\n", correct, total);
     Output ("Total score=%d Max=%d\n", total_score, max_score);
     Output("============================\n");
   }
   Output ("\nTotal nodes = %ld\n", TotalNodes);
}

void Solve (char *file, searchparams_t *searchparams)
/*****************************************************************************
 *
 *
 *
 *****************************************************************************/
{

   searchstats_t searchstats[1];
   epd_iter_t epd_iter[1];
   epd_t epd[1];

   int total, correct, found;
   long TotalNodes;
   char SANmv[SANSZ];


   total = correct = 0;
   TotalNodes = 0;
   EPDIterInit(epd_iter);
   EPDIterStart(epd_iter,file);
   if(searchparams->ST==0){
       searchparams->ST=10;
   }
   while (!(Protocol->state_flags & QUIT) && EPDIterNext(epd_iter,epd)) {
     TTableZero(TTable);  
     PTableZero(PTable);  
     EPDShow(epd);
     total++;

     SearchStatsInit(searchstats);
     Iterate (epd->board,searchparams,searchstats);
     TotalNodes += searchstats->NodeCnt + searchstats->QuiesCnt;
     MoveToSAN(epd->board,searchstats->RootPV,SANmv);
     found = false;
     if(MoveListContains(epd->bm,searchstats->RootPV)){
	 correct+=1;
	 found=true;
     }
     Output ("\nmove=%s ", SANmv);
     Output (found ? "(Correct)\n" : "(Incorrect)\n");
     Output ("Correct=%d Total=%d\n", correct, total);
     Output("============================\n");
   }
   Output ("\nTotal nodes = %ld\n", TotalNodes);
}

